@@ -18,6 +18,8 @@ package model
 
 import (
 	"fmt"
+	"net"
+	"os"
 	"path"
 	"path/filepath"
 
@@ -25,6 +27,7 @@ import (
 
 	"github.com/aws/aws-sdk-go/aws/ec2metadata"
 	"github.com/aws/aws-sdk-go/aws/session"
+	"github.com/aws/aws-sdk-go/service/ec2"
 
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/klog/v2"
@@ -117,8 +120,15 @@ func (b *KubeletBuilder) Build(c *fi.ModelBuilderContext) error {
 
 		if b.HasAPIServer || !b.UseBootstrapTokens() {
 			var kubeconfig fi.Resource
-			if b.HasAPIServer && (b.IsKubernetesGTE("1.19") || b.UseBootstrapTokens()) {
-				kubeconfig, err = b.buildMasterKubeletKubeconfig(c)
+			if b.HasAPIServer {
+				if b.IsKubernetesGTE("1.19") || b.UseBootstrapTokens() {
+					kubeconfig, err = b.buildMasterKubeletKubeconfig(c)
+				} else {
+					kubeconfig = b.BuildIssuedKubeconfig("kubelet", nodetasks.PKIXName{
+						CommonName:   "kubelet",
+						Organization: []string{rbac.NodesGroup},
+					}, c)
+				}
 			} else {
 				kubeconfig, err = b.BuildBootstrapKubeconfig("kubelet", c)
 			}
@@ -199,7 +209,7 @@ func (b *KubeletBuilder) buildSystemdEnvironmentFile(kubeletConfig *kops.Kubelet
 	// would be a degree of freedom we don't have (we'd have to write the config to different files)
 	// We can always add this later if it is needed.
 	if b.Cluster.Spec.CloudConfig != nil {
-		flags += " --cloud-config=" + CloudConfigFilePath
+		flags += " --cloud-config=" + InTreeCloudConfigFilePath
 	}
 
 	if b.UsesSecondaryIP() {
@@ -237,6 +247,10 @@ func (b *KubeletBuilder) buildSystemdEnvironmentFile(kubeletConfig *kops.Kubelet
 		flags += " --tls-private-key-file=" + b.PathSrvKubernetes() + "/kubelet-server.key"
 	}
 
+	if b.Cluster.Spec.IsIPv6Only() {
+		flags += " --node-ip=::"
+	}
+
 	sysconfig := "DAEMON_ARGS=\"" + flags + "\"\n"
 	// Makes kubelet read /root/.docker/config.json properly
 	sysconfig = sysconfig + "HOME=\"/root" + "\"\n"
@@ -306,10 +320,6 @@ func (b *KubeletBuilder) buildSystemdService() *nodetasks.Service {
 
 // buildKubeletConfig is responsible for creating the kubelet configuration
 func (b *KubeletBuilder) buildKubeletConfig() (*kops.KubeletConfigSpec, error) {
-	if b.InstanceGroup == nil {
-		klog.Fatalf("InstanceGroup was not set")
-	}
-
 	kubeletConfigSpec, err := b.buildKubeletConfigSpec()
 	if err != nil {
 		return nil, fmt.Errorf("error building kubelet config: %v", err)
@@ -429,7 +439,7 @@ func (b *KubeletBuilder) addContainerizedMounter(c *fi.ModelBuilderContext) erro
 // buildKubeletConfigSpec returns the kubeletconfig for the specified instanceGroup
 func (b *KubeletBuilder) buildKubeletConfigSpec() (*kops.KubeletConfigSpec, error) {
 	isMaster := b.IsMaster
-	isAPIServer := b.InstanceGroup.Spec.Role == kops.InstanceGroupRoleAPIServer
+	isAPIServer := b.BootConfig.InstanceGroupRole == kops.InstanceGroupRoleAPIServer
 
 	// Merge KubeletConfig for NodeLabels
 	c := b.NodeupConfig.KubeletConfig
@@ -524,9 +534,7 @@ func (b *KubeletBuilder) buildKubeletConfigSpec() (*kops.KubeletConfigSpec, erro
 	// As of 1.16 we can no longer set critical labels.
 	// kops-controller will set these labels.
 	// For bootstrapping reasons, protokube sets the critical labels for kops-controller to run.
-	if b.Cluster.IsKubernetesGTE("1.16") {
-		c.NodeLabels = nil
-	}
+	c.NodeLabels = nil
 
 	if c.AuthorizationMode == "" && b.Cluster.IsKubernetesGTE("1.19") {
 		c.AuthorizationMode = "Webhook"
@@ -558,15 +566,17 @@ func (b *KubeletBuilder) buildKubeletServingCertificate(c *fi.ModelBuilderContex
 	if b.UseKopsControllerForNodeBootstrap() {
 		name := "kubelet-server"
 		dir := b.PathSrvKubernetes()
-		signer := fi.CertificateIDCA
 
-		nodeName, err := b.NodeName()
+		names, err := b.kubeletNames()
 		if err != nil {
 			return err
 		}
 
 		if !b.HasAPIServer {
-			cert, key := b.GetBootstrapCert(name)
+			cert, key, err := b.GetBootstrapCert(name, fi.CertificateIDCA)
+			if err != nil {
+				return err
+			}
 
 			c.AddTask(&nodetasks.File{
 				Path:           filepath.Join(dir, name+".crt"),
@@ -586,13 +596,14 @@ func (b *KubeletBuilder) buildKubeletServingCertificate(c *fi.ModelBuilderContex
 
 		} else {
 			issueCert := &nodetasks.IssueCert{
-				Name:   name,
-				Signer: signer,
-				Type:   "server",
+				Name:      name,
+				Signer:    fi.CertificateIDCA,
+				KeypairID: b.NodeupConfig.KeypairIDs[fi.CertificateIDCA],
+				Type:      "server",
 				Subject: nodetasks.PKIXName{
-					CommonName: nodeName,
+					CommonName: names[0],
 				},
-				AlternateNames: []string{nodeName},
+				AlternateNames: names,
 			}
 			c.AddTask(issueCert)
 			return issueCert.AddFileTasks(c, dir, name, "", nil)
@@ -601,3 +612,27 @@ func (b *KubeletBuilder) buildKubeletServingCertificate(c *fi.ModelBuilderContex
 	return nil
 
 }
+
+func (b *KubeletBuilder) kubeletNames() ([]string, error) {
+	if kops.CloudProviderID(b.Cluster.Spec.CloudProvider) != kops.CloudProviderAWS {
+		name, err := os.Hostname()
+		if err != nil {
+			return nil, err
+		}
+
+		addrs, _ := net.LookupHost(name)
+
+		return append(addrs, name), nil
+	}
+
+	cloud := b.Cloud.(awsup.AWSCloud)
+
+	result, err := cloud.EC2().DescribeInstances(&ec2.DescribeInstancesInput{
+		InstanceIds: []*string{&b.InstanceID},
+	})
+	if err != nil {
+		return nil, fmt.Errorf("error describing instances: %v", err)
+	}
+
+	return awsup.GetInstanceCertificateNames(result)
+}
