@@ -17,7 +17,9 @@ limitations under the License.
 package networking
 
 import (
+	"errors"
 	"fmt"
+	"os"
 	"path/filepath"
 
 	"golang.org/x/sys/unix"
@@ -50,12 +52,12 @@ func (b *CiliumBuilder) Build(c *fi.ModelBuilderContext) error {
 	}
 
 	if err := b.buildBPFMount(c); err != nil {
-		return err
+		return fmt.Errorf("failed to create bpf mount unit: %w", err)
 	}
 
-	image := "quay.io/cilium/cilium:" + cilium.Version
-
-	b.WarmPullImage(c, image)
+	if err := b.buildCgroup2Mount(c); err != nil {
+		return fmt.Errorf("failed to create cgroupv2 mount unit: %w", err)
+	}
 
 	return nil
 
@@ -97,6 +99,52 @@ WantedBy=multi-user.target
 			Name:       "sys-fs-bpf.mount",
 			Definition: fi.String(unit),
 		}
+		service.InitDefaults()
+		c.AddTask(service)
+	}
+
+	return nil
+}
+
+func (b *CiliumBuilder) buildCgroup2Mount(c *fi.ModelBuilderContext) error {
+
+	cgroupPath := "/run/cilium/cgroupv2"
+
+	var fsdata unix.Statfs_t
+	err := unix.Statfs(cgroupPath, &fsdata)
+
+	// If the path does not exist, systemd will create it
+	if !errors.Is(err, os.ErrNotExist) {
+		return fmt.Errorf("error checking for /run/cilium/cgroupv2: %v", err)
+	}
+
+	CGROUP_FS_MAGIC := uint32(0x63677270)
+
+	alreadyMounted := uint32(fsdata.Type) == CGROUP_FS_MAGIC
+
+	if !alreadyMounted {
+		unit := `
+[Unit]
+Description=Cilium Cgroup2 mounts
+Documentation=http://docs.cilium.io/
+DefaultDependencies=no
+Before=local-fs.target umount.target kubelet.service
+
+[Mount]
+What=cgroup2
+Where=/run/cilium/cgroupv2
+Type=cgroup2
+
+[Install]
+WantedBy=multi-user.target
+`
+
+		service := &nodetasks.Service{
+			Name:         "run-cilium-cgroupv2.mount",
+			Definition:   fi.String(unit),
+			SmartRestart: fi.Bool(false),
+		}
+		service.InitDefaults()
 		c.AddTask(service)
 	}
 
@@ -104,63 +152,53 @@ WantedBy=multi-user.target
 }
 
 func (b *CiliumBuilder) buildCiliumEtcdSecrets(c *fi.ModelBuilderContext) error {
-
-	if b.IsMaster {
-		d := "/etc/kubernetes/pki/etcd-manager-cilium"
-
-		keys := make(map[string]string)
-		keys["etcd-manager-ca"] = "etcd-manager-ca-cilium"
-		keys["etcd-peers-ca"] = "etcd-peers-ca-cilium"
-		keys["etcd-clients-ca"] = "etcd-clients-ca-cilium"
-
-		for fileName, keystoreName := range keys {
-			_, err := b.KeyStore.FindCert(keystoreName)
-			if err != nil {
-				return err
-			}
-
-			if err := b.BuildCertificateTask(c, keystoreName, d+"/"+fileName+".crt", nil); err != nil {
-				return err
-			}
-			if err := b.BuildPrivateKeyTask(c, keystoreName, d+"/"+fileName+".key", nil); err != nil {
-				return err
-			}
-		}
-	}
-
 	name := "etcd-client-cilium"
 	dir := "/etc/kubernetes/pki/cilium"
 	signer := "etcd-clients-ca-cilium"
-	if b.UseKopsControllerForNodeBootstrap() && !b.IsMaster {
-		cert, key := b.GetBootstrapCert(name)
-
-		c.AddTask(&nodetasks.File{
-			Path:           filepath.Join(dir, name+".crt"),
-			Contents:       cert,
-			Type:           nodetasks.FileType_File,
-			Mode:           fi.String("0644"),
-			BeforeServices: []string{"kubelet.service"},
-		})
-
-		c.AddTask(&nodetasks.File{
-			Path:           filepath.Join(dir, name+".key"),
-			Contents:       key,
-			Type:           nodetasks.FileType_File,
-			Mode:           fi.String("0400"),
-			BeforeServices: []string{"kubelet.service"},
-		})
-
-		return b.BuildCertificateTask(c, signer, filepath.Join(dir, "etcd-ca.crt"), nil)
-	} else {
+	c.AddTask(&nodetasks.File{
+		Path:     filepath.Join(dir, "etcd-ca.crt"),
+		Contents: fi.NewStringResource(b.NodeupConfig.CAs[signer]),
+		Type:     nodetasks.FileType_File,
+		Mode:     fi.String("0600"),
+	})
+	if b.HasAPIServer {
 		issueCert := &nodetasks.IssueCert{
-			Name:   name,
-			Signer: signer,
-			Type:   "client",
+			Name:      name,
+			Signer:    signer,
+			KeypairID: b.NodeupConfig.KeypairIDs[signer],
+			Type:      "client",
 			Subject: nodetasks.PKIXName{
 				CommonName: "cilium",
 			},
 		}
 		c.AddTask(issueCert)
-		return issueCert.AddFileTasks(c, dir, name, "etcd-ca", nil)
+		return issueCert.AddFileTasks(c, dir, name, "", nil)
+	} else {
+		if b.UseKopsControllerForNodeBootstrap() {
+			cert, key, err := b.GetBootstrapCert(name, signer)
+			if err != nil {
+				return err
+			}
+
+			c.AddTask(&nodetasks.File{
+				Path:           filepath.Join(dir, name+".crt"),
+				Contents:       cert,
+				Type:           nodetasks.FileType_File,
+				Mode:           fi.String("0644"),
+				BeforeServices: []string{"kubelet.service"},
+			})
+
+			c.AddTask(&nodetasks.File{
+				Path:           filepath.Join(dir, name+".key"),
+				Contents:       key,
+				Type:           nodetasks.FileType_File,
+				Mode:           fi.String("0400"),
+				BeforeServices: []string{"kubelet.service"},
+			})
+
+			return nil
+		} else {
+			return b.BuildCertificatePairTask(c, name, dir, name, nil, []string{"kubelet.service"})
+		}
 	}
 }
