@@ -101,16 +101,24 @@ func (t *ProtokubeBuilder) Build(c *fi.ModelBuilderContext) error {
 
 		// retrieve the etcd peer certificates and private keys from the keystore
 		if !t.UseEtcdManager() && t.UseEtcdTLS() {
-			for _, x := range []string{"etcd", "etcd-peer", "etcd-client"} {
+			for _, x := range []string{"etcd", "etcd-peer"} {
 				if err := t.BuildCertificateTask(c, x, fmt.Sprintf("%s.pem", x), nil); err != nil {
 					return err
 				}
 			}
-			for _, x := range []string{"etcd", "etcd-peer", "etcd-client"} {
-				if err := t.BuildPrivateKeyTask(c, x, fmt.Sprintf("%s-key.pem", x), nil); err != nil {
+			for _, x := range []string{"etcd", "etcd-peer"} {
+				if err := t.BuildLegacyPrivateKeyTask(c, x, fmt.Sprintf("%s-key.pem", x), nil); err != nil {
 					return err
 				}
 			}
+			pathEtcdClient := filepath.Join(t.PathSrvKubernetes(), "kube-apiserver", "etcd-client")
+			if err := t.BuildCertificateTask(c, "etcd-client", pathEtcdClient+".crt", nil); err != nil {
+				return err
+			}
+			if err := t.BuildLegacyPrivateKeyTask(c, "etcd-client", pathEtcdClient+".key", nil); err != nil {
+				return err
+			}
+
 		}
 	}
 
@@ -275,7 +283,7 @@ func (t *ProtokubeBuilder) ProtokubeFlags(k8sVersion semver.Version) (*Protokube
 		if etcdContainerImage != "" {
 			image = etcdContainerImage
 		}
-		assets := assets.NewAssetBuilder(t.Cluster, "")
+		assets := assets.NewAssetBuilder(t.Cluster, false)
 		remapped, err := assets.RemapImage(image)
 		if err != nil {
 			return nil, fmt.Errorf("unable to remap container %q: %v", image, err)
