@@ -45,6 +45,10 @@ type KubeletConfigSpec struct {
 	KubeconfigPath string `json:"kubeconfigPath,omitempty" flag:"kubeconfig"`
 	// RequireKubeconfig indicates a kubeconfig is required
 	RequireKubeconfig *bool `json:"requireKubeconfig,omitempty" flag:"require-kubeconfig"`
+	// LogFormat is the logging format of the kubelet.
+	// Supported values: text, json.
+	// Default: text
+	LogFormat string `json:"logFormat,omitempty" flag:"logging-format" flag-empty:"text"`
 	// LogLevel is the logging level of the kubelet
 	LogLevel *int32 `json:"logLevel,omitempty" flag:"v" flag-empty:"0"`
 	// config is the path to the config file or directory of files
@@ -217,6 +221,8 @@ type KubeletConfigSpec struct {
 	ContainerLogMaxFiles *int32 `json:"containerLogMaxFiles,omitempty" flag:"container-log-max-files"`
 	// EnableCadvisorJsonEndpoints enables cAdvisor json `/spec` and `/stats/*` endpoints. Defaults to False.
 	EnableCadvisorJsonEndpoints *bool `json:"enableCadvisorJsonEndpoints,omitempty" flag:"enable-cadvisor-json-endpoints"`
+	// PodPidsLimit is the maximum number of pids in any pod.
+	PodPidsLimit *int64 `json:"podPidsLimit,omitempty" flag:"pod-max-pids"`
 }
 
 // KubeProxyConfig defines the configuration for a proxy
@@ -269,6 +275,10 @@ type KubeAPIServerConfig struct {
 	Image string `json:"image,omitempty"`
 	// DisableBasicAuth removes the --basic-auth-file flag
 	DisableBasicAuth *bool `json:"disableBasicAuth,omitempty"`
+	// LogFormat is the logging format of the api.
+	// Supported values: text, json.
+	// Default: text
+	LogFormat string `json:"logFormat,omitempty" flag:"logging-format" flag-empty:"text"`
 	// LogLevel is the logging level of the api
 	LogLevel int32 `json:"logLevel,omitempty" flag:"v" flag-empty:"0"`
 	// CloudProvider is the name of the cloudProvider we are using, aws, gce etcd
@@ -511,6 +521,10 @@ type KubeAPIServerConfig struct {
 type KubeControllerManagerConfig struct {
 	// Master is the url for the kube api master
 	Master string `json:"master,omitempty" flag:"master"`
+	// LogFormat is the logging format of the controler manager.
+	// Supported values: text, json.
+	// Default: text
+	LogFormat string `json:"logFormat,omitempty" flag:"logging-format" flag-empty:"text"`
 	// LogLevel is the defined logLevel
 	LogLevel int32 `json:"logLevel,omitempty" flag:"v" flag-empty:"0"`
 	// ServiceAccountPrivateKeyFile is the location of the private key for service account token signing.
@@ -590,10 +604,14 @@ type KubeControllerManagerConfig struct {
 	ExperimentalClusterSigningDuration *metav1.Duration `json:"experimentalClusterSigningDuration,omitempty" flag:"experimental-cluster-signing-duration"`
 	// FeatureGates is set of key=value pairs that describe feature gates for alpha/experimental features.
 	FeatureGates map[string]string `json:"featureGates,omitempty" flag:"feature-gates"`
+	// TLSCertFile is the file containing the TLS server certificate.
+	TLSCertFile *string `json:"tlsCertFile,omitempty" flag:"tls-cert-file"`
 	// TLSCipherSuites indicates the allowed TLS cipher suite
 	TLSCipherSuites []string `json:"tlsCipherSuites,omitempty" flag:"tls-cipher-suites"`
 	// TLSMinVersion indicates the minimum TLS version allowed
 	TLSMinVersion string `json:"tlsMinVersion,omitempty" flag:"tls-min-version"`
+	// TLSPrivateKeyFile is the file containing the private key for the TLS server certificate.
+	TLSPrivateKeyFile string `json:"tlsPrivateKeyFile,omitempty" flag:"tls-private-key-file"`
 	// MinResyncPeriod indicates the resync period in reflectors.
 	// The resync period will be random between MinResyncPeriod and 2*MinResyncPeriod. (default 12h0m0s)
 	MinResyncPeriod string `json:"minResyncPeriod,omitempty" flag:"min-resync-period"`
@@ -662,6 +680,10 @@ type CloudControllerManagerConfig struct {
 type KubeSchedulerConfig struct {
 	// Master is a url to the kube master
 	Master string `json:"master,omitempty" flag:"master"`
+	// LogFormat is the logging format of the scheduler.
+	// Supported values: text, json.
+	// Default: text
+	LogFormat string `json:"logFormat,omitempty" flag:"logging-format" flag-empty:"text"`
 	// LogLevel is the logging level
 	LogLevel int32 `json:"logLevel,omitempty" flag:"v"`
 	// Image is the docker image to use
@@ -690,6 +712,10 @@ type KubeSchedulerConfig struct {
 
 	// EnableProfiling enables profiling via web interface host:port/debug/pprof/
 	EnableProfiling *bool `json:"enableProfiling,omitempty" flag:"profiling"`
+	// TLSCertFile is the file containing the TLS server certificate.
+	TLSCertFile *string `json:"tlsCertFile,omitempty" flag:"tls-cert-file"`
+	// TLSPrivateKeyFile is the file containing the private key for the TLS server certificate.
+	TLSPrivateKeyFile string `json:"tlsPrivateKeyFile,omitempty" flag:"tls-private-key-file"`
 }
 
 // LeaderElectionConfiguration defines the configuration of leader election
@@ -721,14 +747,15 @@ type LeaderElectionConfiguration struct {
 
 // OpenstackLoadbalancerConfig defines the config for a neutron loadbalancer
 type OpenstackLoadbalancerConfig struct {
-	Method            *string `json:"method,omitempty"`
-	Provider          *string `json:"provider,omitempty"`
-	UseOctavia        *bool   `json:"useOctavia,omitempty"`
-	FloatingNetwork   *string `json:"floatingNetwork,omitempty"`
-	FloatingNetworkID *string `json:"floatingNetworkID,omitempty"`
-	FloatingSubnet    *string `json:"floatingSubnet,omitempty"`
-	SubnetID          *string `json:"subnetID,omitempty"`
-	ManageSecGroups   *bool   `json:"manageSecurityGroups,omitempty"`
+	Method                *string `json:"method,omitempty"`
+	Provider              *string `json:"provider,omitempty"`
+	UseOctavia            *bool   `json:"useOctavia,omitempty"`
+	FloatingNetwork       *string `json:"floatingNetwork,omitempty"`
+	FloatingNetworkID     *string `json:"floatingNetworkID,omitempty"`
+	FloatingSubnet        *string `json:"floatingSubnet,omitempty"`
+	SubnetID              *string `json:"subnetID,omitempty"`
+	ManageSecGroups       *bool   `json:"manageSecurityGroups,omitempty"`
+	EnableIngressHostname *bool   `json:"enableIngressHostname,omitempty"`
 }
 
 type OpenstackBlockStorageConfig struct {
@@ -761,6 +788,12 @@ type OpenstackNetwork struct {
 	AvailabilityZoneHints []*string `json:"availabilityZoneHints,omitempty"`
 }
 
+// OpenstackMetadata defines config for metadata service related settings
+type OpenstackMetadata struct {
+	// ConfigDrive specifies to use config drive for retrieving user data instead of the metadata service when launching instances
+	ConfigDrive *bool `json:"configDrive,omitempty"`
+}
+
 // OpenstackConfiguration defines cloud config elements for the openstack cloud provider
 type OpenstackConfiguration struct {
 	Loadbalancer       *OpenstackLoadbalancerConfig `json:"loadbalancer,omitempty"`
@@ -769,6 +802,7 @@ type OpenstackConfiguration struct {
 	BlockStorage       *OpenstackBlockStorageConfig `json:"blockStorage,omitempty"`
 	InsecureSkipVerify *bool                        `json:"insecureSkipVerify,omitempty"`
 	Network            *OpenstackNetwork            `json:"network,omitempty"`
+	Metadata           *OpenstackMetadata           `json:"metadata,omitempty"`
 }
 
 // AzureConfiguration defines Azure specific cluster configuration.
@@ -803,6 +837,8 @@ type CloudConfiguration struct {
 	Multizone          *bool   `json:"multizone,omitempty"`
 	NodeTags           *string `json:"nodeTags,omitempty"`
 	NodeInstancePrefix *string `json:"nodeInstancePrefix,omitempty"`
+	// NodeIPFamilies controls the IP families reported for each node (AWS only).
+	NodeIPFamilies []string `json:"nodeIPFamilies,omitempty"`
 	// GCEServiceAccount specifies the service account with which the GCE VM runs
 	GCEServiceAccount string `json:"gceServiceAccount,omitempty"`
 	// AWS cloud-config options
@@ -869,6 +905,12 @@ type NodeTerminationHandlerConfig struct {
 	// EnableScheduledEventDraining makes node termination handler drain nodes before the maintenance window starts for an EC2 instance scheduled event.
 	// Default: false
 	EnableScheduledEventDraining *bool `json:"enableScheduledEventDraining,omitempty"`
+	// EnableRebalanceMonitoring makes node termination handler cordon nodes when the rebalance recommendation notice is received
+	// Default: false
+	EnableRebalanceMonitoring *bool `json:"enableRebalanceMonitoring,omitempty"`
+	// EnableRebalanceDraining makes node termination handler drain nodes when the rebalance recommendation notice is received
+	// Default: false
+	EnableRebalanceDraining *bool `json:"enableRebalanceDraining,omitempty"`
 
 	// EnablePrometheusMetrics enables the "/metrics" endpoint.
 	EnablePrometheusMetrics *bool `json:"prometheusEnable,omitempty"`
@@ -887,6 +929,28 @@ type NodeTerminationHandlerConfig struct {
 	CPURequest *resource.Quantity `json:"cpuRequest,omitempty"`
 }
 
+// NodeProblemDetector determines the node problem detector configuration.
+type NodeProblemDetectorConfig struct {
+	// Enabled enables the NodeProblemDetector.
+	// Default: false
+	Enabled *bool `json:"enabled,omitempty"`
+	// Image is the NodeProblemDetector docker container used.
+	Image *string `json:"image,omitempty"`
+
+	// MemoryRequest of NodeProblemDetector container.
+	// Default: 80Mi
+	MemoryRequest *resource.Quantity `json:"memoryRequest,omitempty"`
+	// CPURequest of NodeProblemDetector container.
+	// Default: 10m
+	CPURequest *resource.Quantity `json:"cpuRequest,omitempty"`
+	// MemoryLimit of NodeProblemDetector container.
+	// Default: 80Mi
+	MemoryLimit *resource.Quantity `json:"memoryLimit,omitempty"`
+	// CPULimit of NodeProblemDetector container.
+	// Default: 10m
+	CPULimit *resource.Quantity `json:"cpuLimit,omitempty"`
+}
+
 // ClusterAutoscalerConfig determines the cluster autoscaler configuration.
 type ClusterAutoscalerConfig struct {
 	// Enabled enables the cluster autoscaler.
