@@ -288,7 +288,7 @@ func TestSetClusterFields(t *testing.T) {
 				Spec: kops.ClusterSpec{
 					Networking: &kops.NetworkingSpec{
 						Cilium: &kops.CiliumNetworkingSpec{
-							DisableMasquerade: true,
+							DisableMasquerade: fi.Bool(true),
 						},
 					},
 				},
@@ -367,7 +367,7 @@ func TestSetCiliumFields(t *testing.T) {
 						Cilium: &kops.CiliumNetworkingSpec{
 							Ipam:              "eni",
 							EnableNodePort:    true,
-							DisableMasquerade: true,
+							DisableMasquerade: fi.Bool(true),
 						},
 					},
 				},
