@@ -68,6 +68,16 @@ func TestRootVolumeOptimizationFlag(t *testing.T) {
 				InstanceGroups:  igs,
 			},
 		},
+		BootstrapScriptBuilder: &model.BootstrapScriptBuilder{
+			Lifecycle: fi.LifecycleSync,
+			Cluster: &kops.Cluster{
+				Spec: kops.ClusterSpec{
+					CloudProvider:     "aws",
+					Networking:        &kops.NetworkingSpec{},
+					KubernetesVersion: "1.20.0",
+				},
+			},
+		},
 		Cluster: cluster,
 	}
 
@@ -82,6 +92,16 @@ func TestRootVolumeOptimizationFlag(t *testing.T) {
 		Type:    "ca",
 	}
 	c.AddTask(caTask)
+	for _, keypair := range []string{
+		"etcd-clients-ca",
+	} {
+		task := &fitasks.Keypair{
+			Name:    fi.String(keypair),
+			Subject: "cn=" + keypair,
+			Type:    "ca",
+		}
+		c.AddTask(task)
+	}
 
 	if err := b.Build(c); err != nil {
 		t.Fatalf("error from Build: %v", err)
@@ -112,7 +132,7 @@ func TestAPIServerAdditionalSecurityGroupsWithNLB(t *testing.T) {
 		_roleCount
 	)
 	igs := make([]*kops.InstanceGroup, _roleCount)
-	// NB: (*AutoscalingGroupModelBuilder).buildLaunchConfigurationTask expects there to be at least
+	// NB: (*AutoscalingGroupModelBuilder).buildLaunchTemplateTask expects there to be at least
 	// one subnet specified in each InstanceGroup.
 	subnets := []string{cluster.Spec.Subnets[0].Name}
 	igs[roleBastion] = &kops.InstanceGroup{
@@ -154,6 +174,14 @@ func TestAPIServerAdditionalSecurityGroupsWithNLB(t *testing.T) {
 				InstanceGroups:  igs,
 			},
 		},
+		BootstrapScriptBuilder: &model.BootstrapScriptBuilder{
+			KopsModelContext: &model.KopsModelContext{
+				IAMModelContext: iam.IAMModelContext{Cluster: cluster},
+				InstanceGroups:  igs,
+			},
+			Lifecycle: fi.LifecycleSync,
+			Cluster:   cluster,
+		},
 		Cluster: cluster,
 	}
 
@@ -168,6 +196,34 @@ func TestAPIServerAdditionalSecurityGroupsWithNLB(t *testing.T) {
 		Type:    "ca",
 	}
 	c.AddTask(caTask)
+	for _, keypair := range []string{
+		"apiserver-aggregator-ca",
+		"etcd-clients-ca",
+		"etcd-manager-ca-events",
+		"etcd-manager-ca-main",
+		"etcd-peers-ca-events",
+		"etcd-peers-ca-main",
+		"service-account",
+	} {
+		task := &fitasks.Keypair{
+			Name:    fi.String(keypair),
+			Subject: "cn=" + keypair,
+			Type:    "ca",
+		}
+		c.AddTask(task)
+	}
+	for _, keypair := range []string{
+		"kubelet",
+		"kube-proxy",
+	} {
+		task := &fitasks.Keypair{
+			Name:    fi.String(keypair),
+			Subject: "cn=" + keypair,
+			Signer:  caTask,
+			Type:    "client",
+		}
+		c.AddTask(task)
+	}
 
 	b.Build(c)
 
