@@ -65,13 +65,17 @@ var (
 			name:    "RebalanceRecommendation",
 			pattern: `{"source": ["aws.ec2"],"detail-type": ["EC2 Instance Rebalance Recommendation"]}`,
 		},
+		{
+			name:    "InstanceStateChange",
+			pattern: `{"source": ["aws.ec2"],"detail-type": ["EC2 Instance State-change Notification"]}`,
+		},
 	}
 )
 
 type NodeTerminationHandlerBuilder struct {
 	*AWSModelContext
 
-	Lifecycle *fi.Lifecycle
+	Lifecycle fi.Lifecycle
 }
 
 func (b *NodeTerminationHandlerBuilder) Build(c *fi.ModelBuilderContext) error {
