@@ -17,7 +17,6 @@ limitations under the License.
 package awsmodel
 
 import (
-	"k8s.io/kops/pkg/model/iam"
 	"k8s.io/kops/upup/pkg/fi"
 	"k8s.io/kops/upup/pkg/fi/cloudup/awstasks"
 )
@@ -25,8 +24,8 @@ import (
 // OIDCProviderBuilder configures IAM OIDC Provider
 type OIDCProviderBuilder struct {
 	*AWSModelContext
-	KeyStore  fi.CAStore
-	Lifecycle *fi.Lifecycle
+	KeyStore  fi.Keystore
+	Lifecycle fi.Lifecycle
 }
 
 var _ fi.ModelBuilder = &OIDCProviderBuilder{}
@@ -42,11 +41,6 @@ func (b *OIDCProviderBuilder) Build(c *fi.ModelBuilderContext) error {
 		return nil
 	}
 
-	serviceAccountIssuer, err := iam.ServiceAccountIssuer(&b.Cluster.Spec)
-	if err != nil {
-		return err
-	}
-
 	fingerprints := getFingerprints()
 
 	thumbprints := []*string{}
@@ -55,11 +49,16 @@ func (b *OIDCProviderBuilder) Build(c *fi.ModelBuilderContext) error {
 		thumbprints = append(thumbprints, fi.String(fingerprint))
 	}
 
+	audiences := []string{defaultAudience}
+	if b.Cluster.Spec.ServiceAccountIssuerDiscovery.AdditionalAudiences != nil {
+		audiences = append(audiences, b.Cluster.Spec.ServiceAccountIssuerDiscovery.AdditionalAudiences...)
+	}
+
 	c.AddTask(&awstasks.IAMOIDCProvider{
 		Name:        fi.String(b.ClusterName()),
 		Lifecycle:   b.Lifecycle,
-		URL:         fi.String(serviceAccountIssuer),
-		ClientIDs:   []*string{fi.String(defaultAudience)},
+		URL:         b.Cluster.Spec.KubeAPIServer.ServiceAccountIssuer,
+		ClientIDs:   fi.StringSlice(audiences),
 		Tags:        b.CloudTags(b.ClusterName(), false),
 		Thumbprints: thumbprints,
 	})
