@@ -192,7 +192,7 @@ func (b *KubeletOptionsBuilder) BuildOptions(o interface{}) error {
 		}
 
 		// Specify our pause image
-		image := "k8s.gcr.io/pause:3.2"
+		image := "k8s.gcr.io/pause:3.5"
 		var err error
 		if image, err = b.AssetBuilder.RemapImage(image); err != nil {
 			return err
@@ -219,16 +219,15 @@ func (b *KubeletOptionsBuilder) BuildOptions(o interface{}) error {
 			}
 		}
 	}
-	if _, found := clusterSpec.Kubelet.FeatureGates["ExperimentalCriticalPodAnnotation"]; !found {
-		if b.IsKubernetesLT("1.16") {
-			clusterSpec.Kubelet.FeatureGates["ExperimentalCriticalPodAnnotation"] = "true"
-		}
-	}
 
 	// Set systemd as the default cgroup driver for kubelet from k8s 1.20
 	if b.IsKubernetesGTE("1.20") && clusterSpec.Kubelet.CgroupDriver == "" {
 		clusterSpec.Kubelet.CgroupDriver = "systemd"
 	}
 
+	if b.IsKubernetesGTE("1.22") && clusterSpec.Kubelet.ProtectKernelDefaults == nil {
+		clusterSpec.Kubelet.ProtectKernelDefaults = fi.Bool(true)
+	}
+
 	return nil
 }
