@@ -53,20 +53,27 @@ function ensure-install-dir() {
   cd ${INSTALL_DIR}
 }
 
-# Retry a download until we get it. args: name, sha, url1, url2...
+# Retry a download until we get it. args: name, sha, urls
 download-or-bust() {
   local -r file="$1"
   local -r hash="$2"
-  shift 2
+  local -r urls=( $(split-commas "$3") )
+
+  if [[ -f "${file}" ]]; then
+    if ! validate-hash "${file}" "${hash}"; then
+      rm -f "${file}"
+    else
+      return
+    fi
+  fi
 
-  urls=( $* )
   while true; do
     for url in "${urls[@]}"; do
       commands=(
-        "curl -f --ipv4 --compressed -Lo "${file}" --connect-timeout 20 --retry 6 --retry-delay 10"
-        "wget --inet4-only --compression=auto -O "${file}" --connect-timeout=20 --tries=6 --wait=10"
-        "curl -f --ipv4 -Lo "${file}" --connect-timeout 20 --retry 6 --retry-delay 10"
-        "wget --inet4-only -O "${file}" --connect-timeout=20 --tries=6 --wait=10"
+        "curl -f --compressed -Lo "${file}" --connect-timeout 20 --retry 6 --retry-delay 10"
+        "wget --compression=auto -O "${file}" --connect-timeout=20 --tries=6 --wait=10"
+        "curl -f -Lo "${file}" --connect-timeout 20 --retry 6 --retry-delay 10"
+        "wget -O "${file}" --connect-timeout=20 --tries=6 --wait=10"
       )
       for cmd in "${commands[@]}"; do
         echo "Attempting download with: ${cmd} {url}"
@@ -74,15 +81,11 @@ download-or-bust() {
           echo "== Download failed with ${cmd} =="
           continue
         fi
-        if [[ -n "${hash}" ]] && ! validate-hash "${file}" "${hash}"; then
+        if ! validate-hash "${file}" "${hash}"; then
           echo "== Hash validation of ${url} failed. Retrying. =="
           rm -f "${file}"
         else
-          if [[ -n "${hash}" ]]; then
-            echo "== Downloaded ${url} (SHA1 = ${hash}) =="
-          else
-            echo "== Downloaded ${url} =="
-          fi
+          echo "== Downloaded ${url} (SHA256 = ${hash}) =="
           return
         fi
       done
@@ -109,23 +112,6 @@ function split-commas() {
   echo $1 | tr "," "\n"
 }
 
-function try-download-release() {
-  local -r nodeup_urls=( $(split-commas "${NODEUP_URL}") )
-  if [[ -n "${NODEUP_HASH:-}" ]]; then
-    local -r nodeup_hash="${NODEUP_HASH}"
-  else
-  # TODO: Remove?
-    echo "Downloading sha256 (not found in env)"
-    download-or-bust nodeup.sha256 "" "${nodeup_urls[@]/%/.sha256}"
-    local -r nodeup_hash=$(cat nodeup.sha256)
-  fi
-
-  echo "Downloading nodeup (${nodeup_urls[@]})"
-  download-or-bust nodeup "${nodeup_hash}" "${nodeup_urls[@]}"
-
-  chmod +x nodeup
-}
-
 function download-release() {
   case "$(uname -m)" in
   x86_64*|i?86_64*|amd64*)
@@ -142,12 +128,10 @@ function download-release() {
     ;;
   esac
 
-  # In case of failure checking integrity of release, retry.
   cd ${INSTALL_DIR}/bin
-  until try-download-release; do
-    sleep 15
-    echo "Couldn't download release. Retrying..."
-  done
+  download-or-bust nodeup "${NODEUP_HASH}" "${NODEUP_URL}"
+
+  chmod +x nodeup
 
   echo "Running nodeup"
   # We can't run in the foreground because of https://github.com/docker/docker/issues/23793
@@ -169,14 +153,6 @@ cat > conf/cluster_spec.yaml << '__EOF_CLUSTER_SPEC'
 __EOF_CLUSTER_SPEC
 {{- end }}
 
-{{ if CompressUserData -}}
-echo "{{ GzipBase64 IGSpec }}" | base64 -d | gzip -d > conf/ig_spec.yaml
-{{- else -}}
-cat > conf/ig_spec.yaml << '__EOF_IG_SPEC'
-{{ IGSpec }}
-__EOF_IG_SPEC
-{{- end }}
-
 {{ if CompressUserData -}}
 echo "{{ GzipBase64 KubeEnv }}" | base64 -d | gzip -d > conf/kube_env.yaml
 {{- else -}}
