@@ -246,32 +246,6 @@ func addUntaggedRouteTables(cloud awsup.AWSCloud, clusterName string, resources
 	return nil
 }
 
-// FindAutoscalingLaunchConfiguration finds an AWS launch configuration given its name
-func FindAutoscalingLaunchConfiguration(cloud awsup.AWSCloud, name string) (*autoscaling.LaunchConfiguration, error) {
-	klog.V(2).Infof("Retrieving Autoscaling LaunchConfigurations %q", name)
-
-	var results []*autoscaling.LaunchConfiguration
-
-	request := &autoscaling.DescribeLaunchConfigurationsInput{
-		LaunchConfigurationNames: []*string{&name},
-	}
-	err := cloud.Autoscaling().DescribeLaunchConfigurationsPages(request, func(p *autoscaling.DescribeLaunchConfigurationsOutput, lastPage bool) bool {
-		results = append(results, p.LaunchConfigurations...)
-		return true
-	})
-	if err != nil {
-		return nil, fmt.Errorf("error listing autoscaling LaunchConfigurations: %v", err)
-	}
-
-	if len(results) == 0 {
-		return nil, nil
-	}
-	if len(results) != 1 {
-		return nil, fmt.Errorf("found multiple LaunchConfigurations with name %q", name)
-	}
-	return results[0], nil
-}
-
 func matchesElbTags(tags map[string]string, actual []*elb.Tag) bool {
 	for k, v := range tags {
 		found := false
@@ -420,7 +394,7 @@ func ListInstances(cloud fi.Cloud, clusterName string) ([]*resources.Resource, e
 					case "terminated", "shutting-down":
 						continue
 
-					case "running", "stopped":
+					case "running", "stopped", "pending":
 						// We need to delete
 						klog.V(4).Infof("instance %q has state=%q", id, stateName)
 
@@ -1801,7 +1775,9 @@ func ListRoute53Records(cloud fi.Cloud, clusterName string) ([]*resources.Resour
 		}
 		err := c.Route53().ListResourceRecordSetsPages(request, func(p *route53.ListResourceRecordSetsOutput, lastPage bool) bool {
 			for _, rrs := range p.ResourceRecordSets {
-				if aws.StringValue(rrs.Type) != "A" {
+				if aws.StringValue(rrs.Type) != "A" &&
+					aws.StringValue(rrs.Type) != "AAAA" &&
+					aws.StringValue(rrs.Type) != "TXT" {
 					continue
 				}
 
@@ -1827,7 +1803,7 @@ func ListRoute53Records(cloud fi.Cloud, clusterName string) ([]*resources.Resour
 
 				resourceTracker := &resources.Resource{
 					Name:     aws.StringValue(rrs.Name),
-					ID:       hostedZoneID + "/" + aws.StringValue(rrs.Name),
+					ID:       hostedZoneID + "/" + aws.StringValue(rrs.Type) + "/" + aws.StringValue(rrs.Name),
 					Type:     "route53-record",
 					GroupKey: hostedZoneID,
 					GroupDeleter: func(cloud fi.Cloud, resourceTrackers []*resources.Resource) error {
@@ -2021,23 +1997,35 @@ func DeleteIAMInstanceProfile(cloud fi.Cloud, r *resources.Resource) error {
 func ListIAMInstanceProfiles(cloud fi.Cloud, clusterName string) ([]*resources.Resource, error) {
 	c := cloud.(awsup.AWSCloud)
 
-	remove := make(map[string]bool)
-	remove["masters."+clusterName] = true
-	remove["nodes."+clusterName] = true
-	remove["bastions."+clusterName] = true
-
+	var getProfileErr error
 	var profiles []*iam.InstanceProfile
+	ownershipTag := "kubernetes.io/cluster/" + clusterName
 
 	request := &iam.ListInstanceProfilesInput{}
 	err := c.IAM().ListInstanceProfilesPages(request, func(p *iam.ListInstanceProfilesOutput, lastPage bool) bool {
 		for _, p := range p.InstanceProfiles {
 			name := aws.StringValue(p.InstanceProfileName)
-			if remove[name] {
-				profiles = append(profiles, p)
+			if !strings.HasSuffix(name, "."+clusterName) {
+				continue
+			}
+
+			getRequest := &iam.GetInstanceProfileInput{InstanceProfileName: p.InstanceProfileName}
+			profileOutput, err := c.IAM().GetInstanceProfile(getRequest)
+			if err != nil {
+				getProfileErr = fmt.Errorf("calling IAM GetInstanceProfile on %s: %w", name, err)
+				return false
+			}
+			for _, tag := range profileOutput.InstanceProfile.Tags {
+				if fi.StringValue(tag.Key) == ownershipTag && fi.StringValue(tag.Value) == "owned" {
+					profiles = append(profiles, p)
+				}
 			}
 		}
 		return true
 	})
+	if getProfileErr != nil {
+		return nil, getProfileErr
+	}
 	if err != nil {
 		return nil, fmt.Errorf("error listing IAM instance profiles: %v", err)
 	}
