@@ -17,9 +17,6 @@ limitations under the License.
 package awstasks
 
 import (
-	"encoding/base64"
-
-	"k8s.io/kops/pkg/featureflag"
 	"k8s.io/kops/upup/pkg/fi"
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
 	"k8s.io/kops/upup/pkg/fi/cloudup/terraform"
@@ -31,6 +28,8 @@ type terraformLaunchTemplateNetworkInterface struct {
 	AssociatePublicIPAddress *bool `json:"associate_public_ip_address,omitempty" cty:"associate_public_ip_address"`
 	// DeleteOnTermination indicates whether the network interface should be destroyed on instance termination.
 	DeleteOnTermination *bool `json:"delete_on_termination,omitempty" cty:"delete_on_termination"`
+	// Ipv6AddressCount is the number of IPv6 addresses to assign with the primary network interface.
+	Ipv6AddressCount *int64 `json:"ipv6_address_count,omitempty" cty:"ipv6_address_count"`
 	// SecurityGroups is a list of security group ids.
 	SecurityGroups []*terraformWriter.Literal `json:"security_groups,omitempty" cty:"security_groups"`
 }
@@ -124,6 +123,8 @@ type terraformLaunchTemplateInstanceMetadata struct {
 	HTTPPutResponseHopLimit *int64 `json:"http_put_response_hop_limit,omitempty" cty:"http_put_response_hop_limit"`
 	// HTTPTokens is the state of token usage for your instance metadata requests.
 	HTTPTokens *string `json:"http_tokens,omitempty" cty:"http_tokens"`
+	// HTTPProtocolIPv6 enables the IPv6 instance metadata endpoint
+	HTTPProtocolIPv6 *string `json:"http_protocol_ipv6,omitempty" cty:"http_protocol_ipv6"`
 }
 
 type terraformLaunchTemplate struct {
@@ -200,11 +201,13 @@ func (t *LaunchTemplate) RenderTerraform(target *terraform.TerraformTarget, a, e
 			HTTPEndpoint:            fi.String("enabled"),
 			HTTPTokens:              e.HTTPTokens,
 			HTTPPutResponseHopLimit: e.HTTPPutResponseHopLimit,
+			HTTPProtocolIPv6:        e.HTTPProtocolIPv6,
 		},
 		NetworkInterfaces: []*terraformLaunchTemplateNetworkInterface{
 			{
 				AssociatePublicIPAddress: e.AssociatePublicIP,
 				DeleteOnTermination:      fi.Bool(true),
+				Ipv6AddressCount:         e.IPv6AddressCount,
 			},
 		},
 	}
@@ -254,20 +257,9 @@ func (t *LaunchTemplate) RenderTerraform(target *terraform.TerraformTarget, a, e
 			return err
 		}
 		if d != nil {
-			if featureflag.TerraformJSON.Enabled() {
-				b64d := make([]byte, base64.StdEncoding.EncodedLen(len(d)))
-				base64.StdEncoding.Encode(b64d, d)
-				if len(b64d) != 0 {
-					tf.UserData, err = target.AddFileBytes("aws_launch_template", fi.StringValue(e.Name), "user_data", b64d, false)
-					if err != nil {
-						return err
-					}
-				}
-			} else {
-				tf.UserData, err = target.AddFileBytes("aws_launch_template", fi.StringValue(e.Name), "user_data", d, true)
-				if err != nil {
-					return err
-				}
+			tf.UserData, err = target.AddFileBytes("aws_launch_template", fi.StringValue(e.Name), "user_data", d, true)
+			if err != nil {
+				return err
 			}
 		}
 	}
