@@ -148,6 +148,11 @@ type AWSCloud interface {
 	RemoveELBTags(loadBalancerName string, tags map[string]string) error
 	RemoveELBV2Tags(ResourceArn string, tags map[string]string) error
 
+	FindELBByNameTag(findNameTag string) (*elb.LoadBalancerDescription, error)
+	DescribeELBTags(loadBalancerNames []string) (map[string][]*elb.Tag, error)
+	FindELBV2ByNameTag(findNameTag string) (*elbv2.LoadBalancer, error)
+	DescribeELBV2Tags(loadBalancerNames []string) (map[string][]*elbv2.Tag, error)
+
 	// DescribeInstance is a helper that queries for the specified instance by id
 	DescribeInstance(instanceID string) (*ec2.Instance, error)
 
@@ -172,9 +177,6 @@ type AWSCloud interface {
 	// DescribeInstanceType calls ec2.DescribeInstanceType to get information for a particular instance type
 	DescribeInstanceType(instanceType string) (*ec2.InstanceTypeInfo, error)
 
-	// FindClusterStatus gets the status of the cluster as it exists in AWS, inferred from volumes
-	FindClusterStatus(cluster *kops.Cluster) (*kops.ClusterStatus, error)
-
 	// AccountInfo returns the AWS account ID and AWS partition that we are deploying into
 	AccountInfo() (string, string, error)
 }
@@ -253,7 +255,10 @@ func NewAWSCloud(region string, tags map[string]string) (AWSCloud, error) {
 
 		requestLogger := newRequestLogger(2)
 
-		sess, err := session.NewSession(config)
+		sess, err := session.NewSessionWithOptions(session.Options{
+			Config:            *config,
+			SharedConfigState: session.SharedConfigEnable,
+		})
 		if err != nil {
 			return c, err
 		}
@@ -261,7 +266,10 @@ func NewAWSCloud(region string, tags map[string]string) (AWSCloud, error) {
 		c.cf.Handlers.Send.PushFront(requestLogger)
 		c.addHandlers(region, &c.cf.Handlers)
 
-		sess, err = session.NewSession(config)
+		sess, err = session.NewSessionWithOptions(session.Options{
+			Config:            *config,
+			SharedConfigState: session.SharedConfigEnable,
+		})
 		if err != nil {
 			return c, err
 		}
@@ -269,7 +277,10 @@ func NewAWSCloud(region string, tags map[string]string) (AWSCloud, error) {
 		c.ec2.Handlers.Send.PushFront(requestLogger)
 		c.addHandlers(region, &c.ec2.Handlers)
 
-		sess, err = session.NewSession(config)
+		sess, err = session.NewSessionWithOptions(session.Options{
+			Config:            *config,
+			SharedConfigState: session.SharedConfigEnable,
+		})
 		if err != nil {
 			return c, err
 		}
@@ -277,7 +288,10 @@ func NewAWSCloud(region string, tags map[string]string) (AWSCloud, error) {
 		c.iam.Handlers.Send.PushFront(requestLogger)
 		c.addHandlers(region, &c.iam.Handlers)
 
-		sess, err = session.NewSession(config)
+		sess, err = session.NewSessionWithOptions(session.Options{
+			Config:            *config,
+			SharedConfigState: session.SharedConfigEnable,
+		})
 		if err != nil {
 			return c, err
 		}
@@ -285,7 +299,10 @@ func NewAWSCloud(region string, tags map[string]string) (AWSCloud, error) {
 		c.elb.Handlers.Send.PushFront(requestLogger)
 		c.addHandlers(region, &c.elb.Handlers)
 
-		sess, err = session.NewSession(config)
+		sess, err = session.NewSessionWithOptions(session.Options{
+			Config:            *config,
+			SharedConfigState: session.SharedConfigEnable,
+		})
 		if err != nil {
 			return c, err
 		}
@@ -293,7 +310,10 @@ func NewAWSCloud(region string, tags map[string]string) (AWSCloud, error) {
 		c.elbv2.Handlers.Send.PushFront(requestLogger)
 		c.addHandlers(region, &c.elbv2.Handlers)
 
-		sess, err = session.NewSession(config)
+		sess, err = session.NewSessionWithOptions(session.Options{
+			Config:            *config,
+			SharedConfigState: session.SharedConfigEnable,
+		})
 		if err != nil {
 			return c, err
 		}
@@ -301,7 +321,10 @@ func NewAWSCloud(region string, tags map[string]string) (AWSCloud, error) {
 		c.sts.Handlers.Send.PushFront(requestLogger)
 		c.addHandlers(region, &c.sts.Handlers)
 
-		sess, err = session.NewSession(config)
+		sess, err = session.NewSessionWithOptions(session.Options{
+			Config:            *config,
+			SharedConfigState: session.SharedConfigEnable,
+		})
 		if err != nil {
 			return c, err
 		}
@@ -309,7 +332,10 @@ func NewAWSCloud(region string, tags map[string]string) (AWSCloud, error) {
 		c.autoscaling.Handlers.Send.PushFront(requestLogger)
 		c.addHandlers(region, &c.autoscaling.Handlers)
 
-		sess, err = session.NewSession(config)
+		sess, err = session.NewSessionWithOptions(session.Options{
+			Config:            *config,
+			SharedConfigState: session.SharedConfigEnable,
+		})
 		if err != nil {
 			return c, err
 		}
@@ -324,7 +350,10 @@ func NewAWSCloud(region string, tags map[string]string) (AWSCloud, error) {
 			}
 		}
 
-		sess, err = session.NewSession(config)
+		sess, err = session.NewSessionWithOptions(session.Options{
+			Config:            *config,
+			SharedConfigState: session.SharedConfigEnable,
+		})
 		if err != nil {
 			return c, err
 		}
@@ -332,7 +361,10 @@ func NewAWSCloud(region string, tags map[string]string) (AWSCloud, error) {
 		c.sqs.Handlers.Send.PushFront(requestLogger)
 		c.addHandlers(region, &c.sqs.Handlers)
 
-		sess, err = session.NewSession(config)
+		sess, err = session.NewSessionWithOptions(session.Options{
+			Config:            *config,
+			SharedConfigState: session.SharedConfigEnable,
+		})
 		if err != nil {
 			return c, err
 		}
@@ -1387,6 +1419,183 @@ func (c *awsCloudImplementation) AddTags(name *string, tags map[string]string) {
 	}
 }
 
+func (c *awsCloudImplementation) FindELBByNameTag(findNameTag string) (*elb.LoadBalancerDescription, error) {
+	return findELBByNameTag(c, findNameTag)
+}
+
+func findELBByNameTag(c AWSCloud, findNameTag string) (*elb.LoadBalancerDescription, error) {
+	// TODO: Any way around this?
+	klog.V(2).Infof("Listing all ELBs for findLoadBalancerByNameTag")
+
+	request := &elb.DescribeLoadBalancersInput{}
+	// ELB DescribeTags has a limit of 20 names, so we set the page size here to 20 also
+	request.PageSize = aws.Int64(20)
+
+	var found []*elb.LoadBalancerDescription
+
+	var innerError error
+	err := c.ELB().DescribeLoadBalancersPages(request, func(p *elb.DescribeLoadBalancersOutput, lastPage bool) bool {
+		if len(p.LoadBalancerDescriptions) == 0 {
+			return true
+		}
+
+		// TODO: Filter by cluster?
+
+		var names []string
+		nameToELB := make(map[string]*elb.LoadBalancerDescription)
+		for _, elb := range p.LoadBalancerDescriptions {
+			name := aws.StringValue(elb.LoadBalancerName)
+			nameToELB[name] = elb
+			names = append(names, name)
+		}
+
+		tagMap, err := c.DescribeELBTags(names)
+		if err != nil {
+			innerError = err
+			return false
+		}
+
+		for loadBalancerName, tags := range tagMap {
+			name, foundNameTag := FindELBTag(tags, "Name")
+			if !foundNameTag || name != findNameTag {
+				continue
+			}
+
+			elb := nameToELB[loadBalancerName]
+			found = append(found, elb)
+		}
+		return true
+	})
+	if err != nil {
+		return nil, fmt.Errorf("error describing LoadBalancers: %v", err)
+	}
+	if innerError != nil {
+		return nil, fmt.Errorf("error describing LoadBalancers: %v", innerError)
+	}
+
+	if len(found) == 0 {
+		return nil, nil
+	}
+
+	if len(found) != 1 {
+		return nil, fmt.Errorf("Found multiple ELBs with Name %q", findNameTag)
+	}
+
+	return found[0], nil
+}
+
+func (c *awsCloudImplementation) DescribeELBTags(loadBalancerNames []string) (map[string][]*elb.Tag, error) {
+	return describeELBTags(c, loadBalancerNames)
+}
+
+func describeELBTags(c AWSCloud, loadBalancerNames []string) (map[string][]*elb.Tag, error) {
+	// TODO: Filter by cluster?
+
+	request := &elb.DescribeTagsInput{}
+	request.LoadBalancerNames = aws.StringSlice(loadBalancerNames)
+
+	// TODO: Cache?
+	klog.V(2).Infof("Querying ELB tags for %s", loadBalancerNames)
+	response, err := c.ELB().DescribeTags(request)
+	if err != nil {
+		return nil, err
+	}
+
+	tagMap := make(map[string][]*elb.Tag)
+	for _, tagset := range response.TagDescriptions {
+		tagMap[aws.StringValue(tagset.LoadBalancerName)] = tagset.Tags
+	}
+	return tagMap, nil
+}
+
+func (c *awsCloudImplementation) FindELBV2ByNameTag(findNameTag string) (*elbv2.LoadBalancer, error) {
+	return findELBV2ByNameTag(c, findNameTag)
+}
+
+func findELBV2ByNameTag(c AWSCloud, findNameTag string) (*elbv2.LoadBalancer, error) {
+	// TODO: Any way around this?
+	klog.V(2).Infof("Listing all NLBs for findNetworkLoadBalancerByNameTag")
+
+	request := &elbv2.DescribeLoadBalancersInput{}
+	// ELB DescribeTags has a limit of 20 names, so we set the page size here to 20 also
+	request.PageSize = aws.Int64(20)
+
+	var found []*elbv2.LoadBalancer
+
+	var innerError error
+	err := c.ELBV2().DescribeLoadBalancersPages(request, func(p *elbv2.DescribeLoadBalancersOutput, lastPage bool) bool {
+		if len(p.LoadBalancers) == 0 {
+			return true
+		}
+
+		// TODO: Filter by cluster?
+
+		var arns []string
+		arnToELB := make(map[string]*elbv2.LoadBalancer)
+		for _, elb := range p.LoadBalancers {
+			arn := aws.StringValue(elb.LoadBalancerArn)
+			arnToELB[arn] = elb
+			arns = append(arns, arn)
+		}
+
+		tagMap, err := c.DescribeELBV2Tags(arns)
+		if err != nil {
+			innerError = err
+			return false
+		}
+
+		for loadBalancerArn, tags := range tagMap {
+			name, foundNameTag := FindELBV2Tag(tags, "Name")
+			if !foundNameTag || name != findNameTag {
+				continue
+			}
+			elb := arnToELB[loadBalancerArn]
+			found = append(found, elb)
+		}
+		return true
+	})
+	if err != nil {
+		return nil, fmt.Errorf("error describing LoadBalancers: %v", err)
+	}
+	if innerError != nil {
+		return nil, fmt.Errorf("error describing LoadBalancers: %v", innerError)
+	}
+
+	if len(found) == 0 {
+		return nil, nil
+	}
+
+	if len(found) != 1 {
+		return nil, fmt.Errorf("Found multiple NLBs with Name %q", findNameTag)
+	}
+
+	return found[0], nil
+}
+
+func (c *awsCloudImplementation) DescribeELBV2Tags(loadBalancerArns []string) (map[string][]*elbv2.Tag, error) {
+	return describeELBV2Tags(c, loadBalancerArns)
+}
+
+func describeELBV2Tags(c AWSCloud, loadBalancerArns []string) (map[string][]*elbv2.Tag, error) {
+	// TODO: Filter by cluster?
+
+	request := &elbv2.DescribeTagsInput{}
+	request.ResourceArns = aws.StringSlice(loadBalancerArns)
+
+	// TODO: Cache?
+	klog.V(2).Infof("Querying ELBV2 api for tags for %s", loadBalancerArns)
+	response, err := c.ELBV2().DescribeTags(request)
+	if err != nil {
+		return nil, err
+	}
+
+	tagMap := make(map[string][]*elbv2.Tag)
+	for _, tagset := range response.TagDescriptions {
+		tagMap[aws.StringValue(tagset.ResourceArn)] = tagset.Tags
+	}
+	return tagMap, nil
+}
+
 func (c *awsCloudImplementation) BuildFilters(name *string) []*ec2.Filter {
 	return buildFilters(c.tags, name)
 }
@@ -1688,6 +1897,42 @@ func findVPCInfo(c AWSCloud, vpcID string) (*fi.VPCInfo, error) {
 	return vpcInfo, nil
 }
 
+func (c *awsCloudImplementation) GetApiIngressStatus(cluster *kops.Cluster) ([]fi.ApiIngressStatus, error) {
+	return getApiIngressStatus(c, cluster)
+}
+
+func getApiIngressStatus(c AWSCloud, cluster *kops.Cluster) ([]fi.ApiIngressStatus, error) {
+	var ingresses []fi.ApiIngressStatus
+	if lbDnsName, err := findDNSName(c, cluster); err != nil {
+		return nil, fmt.Errorf("error finding aws DNSName: %v", err)
+	} else if lbDnsName != "" {
+		ingresses = append(ingresses, fi.ApiIngressStatus{Hostname: lbDnsName})
+	}
+
+	return ingresses, nil
+}
+
+func findDNSName(c AWSCloud, cluster *kops.Cluster) (string, error) {
+	name := "api." + cluster.Name
+	if cluster.Spec.API == nil || cluster.Spec.API.LoadBalancer == nil {
+		return "", nil
+	}
+	if cluster.Spec.API.LoadBalancer.Class == kops.LoadBalancerClassClassic {
+		if lb, err := c.FindELBByNameTag(name); err != nil {
+			return "", fmt.Errorf("error looking for AWS ELB: %v", err)
+		} else if lb != nil {
+			return aws.StringValue(lb.DNSName), nil
+		}
+	} else if cluster.Spec.API.LoadBalancer.Class == kops.LoadBalancerClassNetwork {
+		if lb, err := c.FindELBV2ByNameTag(name); err != nil {
+			return "", fmt.Errorf("error looking for AWS NLB: %v", err)
+		} else if lb != nil {
+			return aws.StringValue(lb.DNSName), nil
+		}
+	}
+	return "", nil
+}
+
 // DefaultInstanceType determines an instance type for the specified cluster & instance group
 func (c *awsCloudImplementation) DefaultInstanceType(cluster *kops.Cluster, ig *kops.InstanceGroup) (string, error) {
 	var candidates []string
@@ -1824,3 +2069,40 @@ func GetRolesInInstanceProfile(c AWSCloud, profileName string) ([]string, error)
 	}
 	return roleNames, nil
 }
+
+// GetInstanceCertificateNames returns the instance hostname and addresses that should go into certificates.
+// The first value is the node name and any additional values are IP addresses.
+func GetInstanceCertificateNames(instances *ec2.DescribeInstancesOutput) (addrs []string, err error) {
+
+	if len(instances.Reservations) != 1 {
+		return nil, fmt.Errorf("too many reservations returned for the single instance-id")
+	}
+
+	if len(instances.Reservations[0].Instances) != 1 {
+		return nil, fmt.Errorf("too many instances returned for the single instance-id")
+	}
+
+	instance := instances.Reservations[0].Instances[0]
+
+	name := *instance.PrivateDnsName
+
+	addrs = append(addrs, name)
+
+	// We only use data for the first interface, and only the first IP
+	for _, iface := range instance.NetworkInterfaces {
+		if iface.Attachment == nil {
+			continue
+		}
+		if *iface.Attachment.DeviceIndex != 0 {
+			continue
+		}
+		addrs = append(addrs, *iface.PrivateIpAddress)
+		if iface.Ipv6Addresses != nil && len(iface.Ipv6Addresses) > 0 {
+			addrs = append(addrs, *iface.Ipv6Addresses[0].Ipv6Address)
+		}
+		if iface.Association != nil && iface.Association.PublicIp != nil {
+			addrs = append(addrs, *iface.Association.PublicIp)
+		}
+	}
+	return addrs, nil
+}
