@@ -29,10 +29,16 @@ import (
 	"k8s.io/kops/pkg/model"
 	"k8s.io/kops/pkg/model/awsmodel"
 	"k8s.io/kops/pkg/model/components/addonmanifests"
+	"k8s.io/kops/pkg/model/components/addonmanifests/awscloudcontrollermanager"
+	"k8s.io/kops/pkg/model/components/addonmanifests/awsebscsidriver"
 	"k8s.io/kops/pkg/model/components/addonmanifests/awsloadbalancercontroller"
+	"k8s.io/kops/pkg/model/components/addonmanifests/clusterautoscaler"
 	"k8s.io/kops/pkg/model/components/addonmanifests/dnscontroller"
+	"k8s.io/kops/pkg/model/components/addonmanifests/externaldns"
+	"k8s.io/kops/pkg/model/components/addonmanifests/nodeterminationhandler"
 	"k8s.io/kops/pkg/model/iam"
 	"k8s.io/kops/pkg/templates"
+	"k8s.io/kops/pkg/wellknownoperators"
 	"k8s.io/kops/upup/pkg/fi"
 	"k8s.io/kops/upup/pkg/fi/fitasks"
 	"k8s.io/kops/upup/pkg/fi/utils"
@@ -42,7 +48,7 @@ import (
 type BootstrapChannelBuilder struct {
 	*model.KopsModelContext
 	ClusterAddons kubemanifest.ObjectList
-	Lifecycle     *fi.Lifecycle
+	Lifecycle     fi.Lifecycle
 	templates     *templates.Templates
 	assetBuilder  *assets.AssetBuilder
 }
@@ -74,7 +80,7 @@ func networkingSelector() map[string]string {
 
 // NewBootstrapChannelBuilder creates a new BootstrapChannelBuilder
 func NewBootstrapChannelBuilder(modelContext *model.KopsModelContext,
-	clusterLifecycle *fi.Lifecycle, assetBuilder *assets.AssetBuilder,
+	clusterLifecycle fi.Lifecycle, assetBuilder *assets.AssetBuilder,
 	templates *templates.Templates,
 	addons kubemanifest.ObjectList,
 ) *BootstrapChannelBuilder {
@@ -99,6 +105,11 @@ func (b *BootstrapChannelBuilder) Build(c *fi.ModelBuilderContext) error {
 	}
 
 	for _, a := range addons.Spec.Addons {
+		// Older versions of channels that may be running on the upgrading cluster requires Version to be set
+		// We hardcode version to a high version to ensure an update is triggered on first run, and from then on
+		// only a hash change will trigger an addon update.
+		a.Version = "9.99.0"
+
 		key := *a.Name
 		if a.Id != "" {
 			key = key + "-" + a.Id
@@ -146,14 +157,63 @@ func (b *BootstrapChannelBuilder) Build(c *fi.ModelBuilderContext) error {
 		})
 	}
 
+	if featureflag.UseAddonOperators.Enabled() {
+		ob := &wellknownoperators.Builder{
+			Cluster: b.Cluster,
+		}
+
+		wellKnownAddons, crds, err := ob.Build()
+		if err != nil {
+			return fmt.Errorf("error building well-known operators: %v", err)
+		}
+
+		for _, a := range wellKnownAddons {
+			key := *a.Spec.Name
+			if a.Spec.Id != "" {
+				key = key + "-" + a.Spec.Id
+			}
+			name := b.Cluster.ObjectMeta.Name + "-addons-" + key
+			manifestPath := "addons/" + *a.Spec.Manifest
+
+			// Go through any transforms that are best expressed as code
+			manifestBytes, err := addonmanifests.RemapAddonManifest(&a.Spec, b.KopsModelContext, b.assetBuilder, a.Manifest)
+			if err != nil {
+				klog.Infof("invalid manifest: %s", string(a.Manifest))
+				return fmt.Errorf("error remapping manifest %s: %v", manifestPath, err)
+			}
+
+			// Trim whitespace
+			manifestBytes = []byte(strings.TrimSpace(string(manifestBytes)))
+
+			rawManifest := string(manifestBytes)
+			klog.V(4).Infof("Manifest %v", rawManifest)
+
+			manifestHash, err := utils.HashString(rawManifest)
+			klog.V(4).Infof("hash %s", manifestHash)
+			if err != nil {
+				return fmt.Errorf("error hashing manifest: %v", err)
+			}
+			a.Spec.ManifestHash = manifestHash
+
+			c.AddTask(&fitasks.ManagedFile{
+				Contents:  fi.NewBytesResource(manifestBytes),
+				Lifecycle: b.Lifecycle,
+				Location:  fi.String(manifestPath),
+				Name:      fi.String(name),
+			})
+
+			addons.Spec.Addons = append(addons.Spec.Addons, &a.Spec)
+		}
+
+		b.ClusterAddons = append(b.ClusterAddons, crds...)
+	}
+
 	if b.ClusterAddons != nil {
 		key := "cluster-addons.kops.k8s.io"
-		version := "0.0.0"
 		location := key + "/default.yaml"
 
 		a := &channelsapi.AddonSpec{
 			Name:     fi.String(key),
-			Version:  fi.String(version),
 			Selector: map[string]string{"k8s-addon": key},
 			Manifest: fi.String(location),
 		}
@@ -205,13 +265,14 @@ func (b *BootstrapChannelBuilder) Build(c *fi.ModelBuilderContext) error {
 }
 
 func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*channelsapi.Addons, error) {
+	serviceAccountRoles := []iam.Subject{}
+
 	addons := &channelsapi.Addons{}
 	addons.Kind = "Addons"
 	addons.ObjectMeta.Name = "bootstrap"
 
 	{
 		key := "kops-controller.addons.k8s.io"
-		version := "1.21.2"
 
 		{
 			location := key + "/k8s-1.16.yaml"
@@ -219,10 +280,8 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 				Name:               fi.String(key),
-				Version:            fi.String(version),
 				Selector:           map[string]string{"k8s-addon": key},
 				Manifest:           fi.String(location),
-				KubernetesVersion:  ">=1.16.0-alpha.0",
 				NeedsRollingUpdate: "control-plane",
 				Id:                 id,
 			})
@@ -236,7 +295,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 		addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 			Name:     fi.String(key),
-			Version:  fi.String(version),
 			Selector: map[string]string{"k8s-addon": key},
 			Manifest: fi.String(location),
 		})
@@ -245,7 +303,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 	// @check if podsecuritypolicies are enabled and if so, push the default kube-system policy
 	if b.Cluster.Spec.KubeAPIServer != nil && b.Cluster.Spec.KubeAPIServer.HasAdmissionController("PodSecurityPolicy") {
 		key := "podsecuritypolicy.addons.k8s.io"
-		version := "0.0.4"
 
 		{
 			location := key + "/k8s-1.12.yaml"
@@ -253,7 +310,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 				Name:     fi.String(key),
-				Version:  fi.String(version),
 				Selector: map[string]string{"k8s-addon": key},
 				Manifest: fi.String(location),
 				Id:       id,
@@ -277,7 +333,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 		{
 			key := "kube-dns.addons.k8s.io"
-			version := "1.15.13-kops.3"
 
 			{
 				location := key + "/k8s-1.12.yaml"
@@ -285,7 +340,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 					Name:     fi.String(key),
-					Version:  fi.String(version),
 					Selector: map[string]string{"k8s-addon": key},
 					Manifest: fi.String(location),
 					Id:       id,
@@ -294,10 +348,9 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 		}
 	}
 
-	if kubeDNS.Provider == "CoreDNS" {
+	if kubeDNS.Provider == "CoreDNS" && !featureflag.UseAddonOperators.Enabled() {
 		{
 			key := "coredns.addons.k8s.io"
-			version := "1.8.3-kops.3"
 
 			{
 				location := key + "/k8s-1.12.yaml"
@@ -305,7 +358,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 					Name:     fi.String(key),
-					Version:  fi.String(version),
 					Selector: map[string]string{"k8s-addon": key},
 					Manifest: fi.String(location),
 					Id:       id,
@@ -330,7 +382,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 	if enableRBACAddon {
 		{
 			key := "rbac.addons.k8s.io"
-			version := "1.8.0"
 
 			{
 				location := key + "/k8s-1.8.yaml"
@@ -338,7 +389,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 					Name:     fi.String(key),
-					Version:  fi.String(version),
 					Selector: map[string]string{"k8s-addon": key},
 					Manifest: fi.String(location),
 					Id:       id,
@@ -352,7 +402,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 		// docs: https://kubernetes.io/docs/reference/access-authn-authz/rbac/#other-component-roles
 		// issue: https://github.com/kubernetes/kops/issues/5176
 		key := "kubelet-api.rbac.addons.k8s.io"
-		version := "v0.0.1"
 
 		{
 			location := key + "/k8s-1.9.yaml"
@@ -360,7 +409,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 				Name:     fi.String(key),
-				Version:  fi.String(version),
 				Selector: map[string]string{"k8s-addon": key},
 				Manifest: fi.String(location),
 				Id:       id,
@@ -375,55 +423,39 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 		addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 			Name:     fi.String(key),
-			Version:  fi.String(version),
 			Selector: map[string]string{"k8s-addon": key},
 			Manifest: fi.String(location),
 		})
 	}
 
-	// @check the dns-controller has not been disabled
-	externalDNS := b.Cluster.Spec.ExternalDNS
-	if externalDNS == nil || !externalDNS.Disable {
-		{
-			key := "dns-controller.addons.k8s.io"
-			version := "1.21.2"
-
+	if b.Cluster.Spec.ExternalDNS == nil || b.Cluster.Spec.ExternalDNS.Provider == kops.ExternalDNSProviderDNSController {
+		// @check the dns-controller has not been disabled
+		externalDNS := b.Cluster.Spec.ExternalDNS
+		if externalDNS == nil || !externalDNS.Disable {
 			{
-				location := key + "/k8s-1.12.yaml"
-				id := "k8s-1.12"
+				key := "dns-controller.addons.k8s.io"
 
-				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-					Name:     fi.String(key),
-					Version:  fi.String(version),
-					Selector: map[string]string{"k8s-addon": key},
-					Manifest: fi.String(location),
-					Id:       id,
-				})
-			}
-		}
+				{
+					location := key + "/k8s-1.12.yaml"
+					id := "k8s-1.12"
 
-		// Generate dns-controller ServiceAccount IAM permissions
-		if b.UseServiceAccountIAM() {
-			awsModelContext := &awsmodel.AWSModelContext{
-				KopsModelContext: b.KopsModelContext,
-			}
-
-			serviceAccountRoles := []iam.Subject{&dnscontroller.ServiceAccount{}}
-			for _, serviceAccountRole := range serviceAccountRoles {
-				iamModelBuilder := &awsmodel.IAMModelBuilder{AWSModelContext: awsModelContext, Lifecycle: b.Lifecycle, Cluster: b.Cluster}
-
-				_, err := iamModelBuilder.BuildServiceAccountRoleTasks(serviceAccountRole, c)
-				if err != nil {
-					return nil, err
+					addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
+						Name:     fi.String(key),
+						Selector: map[string]string{"k8s-addon": key},
+						Manifest: fi.String(location),
+						Id:       id,
+					})
 				}
 			}
-		}
-	}
 
-	if featureflag.EnableExternalDNS.Enabled() {
+			// Generate dns-controller ServiceAccount IAM permissions
+			if b.UseServiceAccountExternalPermissions() {
+				serviceAccountRoles = append(serviceAccountRoles, &dnscontroller.ServiceAccount{})
+			}
+		}
+	} else {
 		{
 			key := "external-dns.addons.k8s.io"
-			version := "0.7.6-kops.1"
 
 			{
 				location := key + "/k8s-1.12.yaml"
@@ -431,12 +463,15 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 					Name:     fi.String(key),
-					Version:  fi.String(version),
 					Selector: map[string]string{"k8s-addon": key},
 					Manifest: fi.String(location),
 					Id:       id,
 				})
 			}
+
+			if b.UseServiceAccountExternalPermissions() {
+				serviceAccountRoles = append(serviceAccountRoles, &externaldns.ServiceAccount{})
+			}
 		}
 	}
 
@@ -445,7 +480,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 	if kubeDNS.Provider == "CoreDNS" && NodeLocalDNS != nil && fi.BoolValue(NodeLocalDNS.Enabled) {
 		{
 			key := "nodelocaldns.addons.k8s.io"
-			version := "1.18.0"
 
 			{
 				location := key + "/k8s-1.12.yaml"
@@ -453,7 +487,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 					Name:     fi.String(key),
-					Version:  fi.String(version),
 					Selector: map[string]string{"k8s-addon": key},
 					Manifest: fi.String(location),
 					Id:       id,
@@ -465,28 +498,29 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 	if b.Cluster.Spec.ClusterAutoscaler != nil && fi.BoolValue(b.Cluster.Spec.ClusterAutoscaler.Enabled) {
 		{
 			key := "cluster-autoscaler.addons.k8s.io"
-			version := "1.19.2"
 
 			{
 				location := key + "/k8s-1.15.yaml"
 				id := "k8s-1.15"
 
 				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-					Name:              fi.String(key),
-					Version:           fi.String(version),
-					Selector:          map[string]string{"k8s-addon": key},
-					Manifest:          fi.String(location),
-					KubernetesVersion: ">=1.15.0",
-					Id:                id,
+					Name:     fi.String(key),
+					Selector: map[string]string{"k8s-addon": key},
+					Manifest: fi.String(location),
+					Id:       id,
 				})
 			}
 		}
+
+		if b.UseServiceAccountExternalPermissions() {
+			serviceAccountRoles = append(serviceAccountRoles, &clusterautoscaler.ServiceAccount{})
+		}
+
 	}
 
 	if b.Cluster.Spec.MetricsServer != nil && fi.BoolValue(b.Cluster.Spec.MetricsServer.Enabled) {
 		{
 			key := "metrics-server.addons.k8s.io"
-			version := "0.4.4"
 
 			{
 				location := key + "/k8s-1.11.yaml"
@@ -494,7 +528,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 					Name:     fi.String(key),
-					Version:  fi.String(version),
 					Selector: map[string]string{"k8s-app": "metrics-server"},
 					Manifest: fi.String(location),
 					Id:       id,
@@ -507,7 +540,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 	if b.Cluster.Spec.CertManager != nil && fi.BoolValue(b.Cluster.Spec.CertManager.Enabled) && (b.Cluster.Spec.CertManager.Managed == nil || fi.BoolValue(b.Cluster.Spec.CertManager.Managed)) {
 		{
 			key := "certmanager.io"
-			version := "1.1.0"
 
 			{
 				location := key + "/k8s-1.16.yaml"
@@ -515,7 +547,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 					Name:     fi.String(key),
-					Version:  fi.String(version),
 					Manifest: fi.String(location),
 					Id:       id,
 				})
@@ -528,7 +559,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 	if nth != nil && fi.BoolValue(nth.Enabled) {
 
 		key := "node-termination-handler.aws"
-		version := "1.7.0"
 
 		{
 			location := key + "/k8s-1.11.yaml"
@@ -536,7 +566,48 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 				Name:     fi.String(key),
-				Version:  fi.String(version),
+				Selector: map[string]string{"k8s-addon": key},
+				Manifest: fi.String(location),
+				Id:       id,
+			})
+		}
+
+		if b.UseServiceAccountExternalPermissions() {
+			serviceAccountRoles = append(serviceAccountRoles, &nodeterminationhandler.ServiceAccount{})
+		}
+	}
+
+	npd := b.Cluster.Spec.NodeProblemDetector
+
+	if npd != nil && fi.BoolValue(npd.Enabled) {
+
+		key := "node-problem-detector.addons.k8s.io"
+
+		{
+			location := key + "/k8s-1.17.yaml"
+			id := "k8s-1.17"
+
+			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
+				Name:     fi.String(key),
+				Selector: map[string]string{"k8s-addon": key},
+				Manifest: fi.String(location),
+				Id:       id,
+			})
+		}
+	}
+
+	nvidia := b.Cluster.Spec.Containerd.NvidiaGPU
+
+	if nvidia != nil && fi.BoolValue(nvidia.Enabled) {
+
+		key := "nvidia.addons.k8s.io"
+
+		{
+			location := key + "/k8s-1.16.yaml"
+			id := "k8s-1.16"
+
+			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
+				Name:     fi.String(key),
 				Selector: map[string]string{"k8s-addon": key},
 				Manifest: fi.String(location),
 				Id:       id,
@@ -547,77 +618,44 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 	if b.Cluster.Spec.AWSLoadBalancerController != nil && fi.BoolValue(b.Cluster.Spec.AWSLoadBalancerController.Enabled) {
 
 		key := "aws-load-balancer-controller.addons.k8s.io"
-		version := "2.1.2"
 
 		{
 			location := key + "/k8s-1.9.yaml"
 			id := "k8s-1.9"
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-				Name:              fi.String(key),
-				Version:           fi.String(version),
-				Selector:          map[string]string{"k8s-addon": key},
-				Manifest:          fi.String(location),
-				KubernetesVersion: ">=1.9.0",
-				Id:                id,
-				NeedsPKI:          true,
+				Name:     fi.String(key),
+				Selector: map[string]string{"k8s-addon": key},
+				Manifest: fi.String(location),
+				Id:       id,
+				NeedsPKI: true,
 			})
 		}
 
 		// Generate aws-load-balancer-controller ServiceAccount IAM permissions
-		if b.UseServiceAccountIAM() {
-			awsModelContext := &awsmodel.AWSModelContext{
-				KopsModelContext: b.KopsModelContext,
-			}
-
-			serviceAccountRoles := []iam.Subject{&awsloadbalancercontroller.ServiceAccount{}}
-			for _, serviceAccountRole := range serviceAccountRoles {
-				iamModelBuilder := &awsmodel.IAMModelBuilder{AWSModelContext: awsModelContext, Lifecycle: b.Lifecycle, Cluster: b.Cluster}
-
-				_, err := iamModelBuilder.BuildServiceAccountRoleTasks(serviceAccountRole, c)
-				if err != nil {
-					return nil, err
-				}
-			}
+		if b.UseServiceAccountExternalPermissions() {
+			serviceAccountRoles = append(serviceAccountRoles, &awsloadbalancercontroller.ServiceAccount{})
 		}
 	}
 
 	if kops.CloudProviderID(b.Cluster.Spec.CloudProvider) == kops.CloudProviderAWS {
 		key := "storage-aws.addons.k8s.io"
-		version := "1.17.0"
 
 		{
 			id := "v1.15.0"
 			location := key + "/" + id + ".yaml"
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-				Name:              fi.String(key),
-				Version:           fi.String(version),
-				Selector:          map[string]string{"k8s-addon": key},
-				Manifest:          fi.String(location),
-				KubernetesVersion: ">=1.15.0",
-				Id:                id,
-			})
-		}
-
-		{
-			id := "v1.7.0"
-			location := key + "/" + id + ".yaml"
-
-			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-				Name:              fi.String(key),
-				Version:           fi.String(version),
-				Selector:          map[string]string{"k8s-addon": key},
-				Manifest:          fi.String(location),
-				KubernetesVersion: "<1.15.0",
-				Id:                id,
+				Name:     fi.String(key),
+				Selector: map[string]string{"k8s-addon": key},
+				Manifest: fi.String(location),
+				Id:       id,
 			})
 		}
 	}
 
 	if kops.CloudProviderID(b.Cluster.Spec.CloudProvider) == kops.CloudProviderDO {
 		key := "digitalocean-cloud-controller.addons.k8s.io"
-		version := "1.8.1-kops.1"
 
 		{
 			id := "k8s-1.8"
@@ -625,7 +663,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 				Name:     fi.String(key),
-				Version:  fi.String(version),
 				Selector: map[string]string{"k8s-addon": key},
 				Manifest: fi.String(location),
 				Id:       id,
@@ -635,7 +672,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 	if kops.CloudProviderID(b.Cluster.Spec.CloudProvider) == kops.CloudProviderGCE {
 		key := "storage-gce.addons.k8s.io"
-		version := "1.7.0"
 
 		{
 			id := "v1.7.0"
@@ -643,7 +679,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 				Name:     fi.String(key),
-				Version:  fi.String(version),
 				Selector: map[string]string{"k8s-addon": key},
 				Manifest: fi.String(location),
 				Id:       id,
@@ -655,40 +690,22 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 		key := "spotinst-kubernetes-cluster-controller.addons.k8s.io"
 
 		{
-			id := "v1.9.0"
+			id := "v1.14.0"
 			location := key + "/" + id + ".yaml"
-			version := "1.0.39"
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 				Name:     fi.String(key),
-				Version:  fi.String(version),
 				Selector: map[string]string{"k8s-addon": key},
 				Manifest: fi.String(location),
 				Id:       id,
 			})
 		}
-
-		{
-			id := "v1.14.0"
-			location := key + "/" + id + ".yaml"
-			version := "1.0.75"
-
-			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-				Name:              fi.String(key),
-				Version:           fi.String(version),
-				Selector:          map[string]string{"k8s-addon": key},
-				Manifest:          fi.String(location),
-				KubernetesVersion: ">=1.14.0",
-				Id:                id,
-			})
-		}
 	}
 
 	// The metadata-proxy daemonset conceals node metadata endpoints in GCE.
 	// It will land on nodes labeled cloud.google.com/metadata-proxy-ready=true
 	if kops.CloudProviderID(b.Cluster.Spec.CloudProvider) == kops.CloudProviderGCE {
 		key := "metadata-proxy.addons.k8s.io"
-		version := "0.1.12"
 
 		{
 			id := "v0.1.12"
@@ -696,7 +713,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 				Name:     fi.String(key),
-				Version:  fi.String(version),
 				Selector: map[string]string{"k8s-addon": key},
 				Manifest: fi.String(location),
 				Id:       id,
@@ -706,7 +722,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 	if b.Cluster.Spec.Networking.Kopeio != nil {
 		key := "networking.kope.io"
-		version := "1.0.20181028-kops.2"
 
 		{
 			location := key + "/k8s-1.12.yaml"
@@ -714,7 +729,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 				Name:     fi.String(key),
-				Version:  fi.String(version),
 				Selector: networkingSelector(),
 				Manifest: fi.String(location),
 				Id:       id,
@@ -724,9 +738,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 	if b.Cluster.Spec.Networking.Weave != nil {
 		key := "networking.weave"
-		versions := map[string]string{
-			"k8s-1.12": "2.8.1-kops.1",
-		}
 
 		{
 			location := key + "/k8s-1.12.yaml"
@@ -734,7 +745,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 				Name:     fi.String(key),
-				Version:  fi.String(versions[id]),
 				Selector: networkingSelector(),
 				Manifest: fi.String(location),
 				Id:       id,
@@ -744,9 +754,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 	if b.Cluster.Spec.Networking.Flannel != nil {
 		key := "networking.flannel"
-		versions := map[string]string{
-			"k8s-1.12": "0.13.0-kops.1",
-		}
 
 		{
 			location := key + "/k8s-1.12.yaml"
@@ -754,7 +761,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 				Name:     fi.String(key),
-				Version:  fi.String(versions[id]),
 				Selector: networkingSelector(),
 				Manifest: fi.String(location),
 				Id:       id,
@@ -764,92 +770,38 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 	if b.Cluster.Spec.Networking.Calico != nil {
 		key := "networking.projectcalico.org"
-		versions := map[string]string{
-			"k8s-1.16": "3.19.1-kops.1",
-		}
-
-		{
-			id := "k8s-1.12"
-			location := key + "/" + id + ".yaml"
-
-			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-				Name:              fi.String(key),
-				Version:           fi.String(versions[id]),
-				Selector:          networkingSelector(),
-				Manifest:          fi.String(location),
-				KubernetesVersion: "<1.16.0",
-				Id:                id,
-			})
-		}
 
 		{
 			id := "k8s-1.16"
 			location := key + "/" + id + ".yaml"
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-				Name:              fi.String(key),
-				Version:           fi.String(versions[id]),
-				Selector:          networkingSelector(),
-				Manifest:          fi.String(location),
-				KubernetesVersion: ">=1.16.0",
-				Id:                id,
+				Name:     fi.String(key),
+				Selector: networkingSelector(),
+				Manifest: fi.String(location),
+				Id:       id,
 			})
 		}
 	}
 
 	if b.Cluster.Spec.Networking.Canal != nil {
 		key := "networking.projectcalico.org.canal"
-		versions := map[string]string{
-			"k8s-1.12": "3.7.5-kops.2",
-			"k8s-1.15": "3.12.2-kops.1",
-			"k8s-1.16": "3.13.4-kops.2",
-		}
-		{
-			id := "k8s-1.12"
-			location := key + "/" + id + ".yaml"
 
-			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-				Name:              fi.String(key),
-				Version:           fi.String(versions[id]),
-				Selector:          networkingSelector(),
-				Manifest:          fi.String(location),
-				KubernetesVersion: "<1.15.0",
-				Id:                id,
-			})
-		}
-		{
-			id := "k8s-1.15"
-			location := key + "/" + id + ".yaml"
-
-			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-				Name:              fi.String(key),
-				Version:           fi.String(versions[id]),
-				Selector:          networkingSelector(),
-				Manifest:          fi.String(location),
-				KubernetesVersion: ">=1.15.0 <1.16.0",
-				Id:                id,
-			})
-		}
 		{
 			id := "k8s-1.16"
 			location := key + "/" + id + ".yaml"
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-				Name:              fi.String(key),
-				Version:           fi.String(versions[id]),
-				Selector:          networkingSelector(),
-				Manifest:          fi.String(location),
-				KubernetesVersion: ">=1.16.0",
-				Id:                id,
+				Name:     fi.String(key),
+				Selector: networkingSelector(),
+				Manifest: fi.String(location),
+				Id:       id,
 			})
 		}
 	}
 
 	if b.Cluster.Spec.Networking.Kuberouter != nil {
 		key := "networking.kuberouter"
-		versions := map[string]string{
-			"k8s-1.12": "1.1.1-kops.1",
-		}
 
 		{
 			location := key + "/k8s-1.12.yaml"
@@ -857,7 +809,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 				Name:     fi.String(key),
-				Version:  fi.String(versions[id]),
 				Selector: networkingSelector(),
 				Manifest: fi.String(location),
 				Id:       id,
@@ -868,37 +819,16 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 	if b.Cluster.Spec.Networking.AmazonVPC != nil {
 		key := "networking.amazon-vpc-routed-eni"
 
-		versions := map[string]string{
-			"k8s-1.12": "1.5.5-kops.1",
-			"k8s-1.16": "1.7.8-kops.1",
-		}
-
-		{
-			id := "k8s-1.12"
-			location := key + "/" + id + ".yaml"
-
-			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-				Name:              fi.String(key),
-				Version:           fi.String(versions[id]),
-				Selector:          networkingSelector(),
-				Manifest:          fi.String(location),
-				KubernetesVersion: "<1.16.0",
-				Id:                id,
-			})
-		}
-
 		{
 			id := "k8s-1.16"
 			location := key + "/" + id + ".yaml"
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 				Name:               fi.String(key),
-				Version:            fi.String(versions[id]),
 				Selector:           networkingSelector(),
 				Manifest:           fi.String(location),
-				KubernetesVersion:  ">=1.16.0",
-				NeedsRollingUpdate: "all",
 				Id:                 id,
+				NeedsRollingUpdate: "all",
 			})
 		}
 	}
@@ -913,7 +843,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 	if b.Cluster.Spec.Authentication != nil {
 		if b.Cluster.Spec.Authentication.Kopeio != nil {
 			key := "authentication.kope.io"
-			version := "1.0.20181028-kops.1"
 
 			{
 				location := key + "/k8s-1.12.yaml"
@@ -921,7 +850,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 					Name:     fi.String(key),
-					Version:  fi.String(version),
 					Selector: authenticationSelector,
 					Manifest: fi.String(location),
 					Id:       id,
@@ -930,9 +858,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 		}
 		if b.Cluster.Spec.Authentication.Aws != nil {
 			key := "authentication.aws"
-			versions := map[string]string{
-				"k8s-1.12": "0.5.1-kops.1",
-			}
 
 			{
 				location := key + "/k8s-1.12.yaml"
@@ -940,7 +865,6 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 					Name:     fi.String(key),
-					Version:  fi.String(versions[id]),
 					Selector: authenticationSelector,
 					Manifest: fi.String(location),
 					Id:       id,
@@ -952,18 +876,15 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 	if kops.CloudProviderID(b.Cluster.Spec.CloudProvider) == kops.CloudProviderOpenstack {
 		{
 			key := "storage-openstack.addons.k8s.io"
-			version := "1.18.0-kops.1"
 
 			id := "k8s-1.16"
 			location := key + "/" + id + ".yaml"
 
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-				Name:              fi.String(key),
-				Version:           fi.String(version),
-				Manifest:          fi.String(location),
-				Selector:          map[string]string{"k8s-addon": key},
-				KubernetesVersion: ">=1.15.0",
-				Id:                id,
+				Name:     fi.String(key),
+				Manifest: fi.String(location),
+				Selector: map[string]string{"k8s-addon": key},
+				Id:       id,
 			})
 		}
 
@@ -971,14 +892,12 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 			// cloudprovider specific out-of-tree controller
 			{
 				key := "openstack.addons.k8s.io"
-				version := "1.13.1-kops.1"
 
 				location := key + "/k8s-1.13.yaml"
 				id := "k8s-1.13-ccm"
 
 				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 					Name:     fi.String(key),
-					Version:  fi.String(version),
 					Manifest: fi.String(location),
 					Selector: map[string]string{"k8s-addon": key},
 					Id:       id,
@@ -987,14 +906,12 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 		} else {
 			{
 				key := "core.addons.k8s.io"
-				version := "1.12.1-kops.1"
 
 				location := key + "/k8s-1.12.yaml"
 				id := "k8s-1.12-ccm"
 
 				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 					Name:     fi.String(key),
-					Version:  fi.String(version),
 					Selector: map[string]string{"k8s-addon": key},
 					Manifest: fi.String(location),
 					Id:       id,
@@ -1005,58 +922,56 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 	if kops.CloudProviderID(b.Cluster.Spec.CloudProvider) == kops.CloudProviderAWS {
 
-		if b.Cluster.Spec.ExternalCloudControllerManager != nil {
+		if b.IsKubernetesGTE("1.18") && b.Cluster.Spec.ExternalCloudControllerManager != nil {
 			key := "aws-cloud-controller.addons.k8s.io"
-			// Version refers to the addon configuration.  The CCM tag is given by
-			// the template function AWSCCMTag()
-			version := "1.18.0-kops.1"
+
 			{
 				id := "k8s-1.18"
 				location := key + "/" + id + ".yaml"
 				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-					Name:              fi.String(key),
-					Version:           fi.String(version),
-					Manifest:          fi.String(location),
-					Selector:          map[string]string{"k8s-addon": key},
-					KubernetesVersion: ">=1.18.0",
-					Id:                id,
+					Name:     fi.String(key),
+					Manifest: fi.String(location),
+					Selector: map[string]string{"k8s-addon": key},
+					Id:       id,
 				})
 			}
+			if b.UseServiceAccountExternalPermissions() {
+				serviceAccountRoles = append(serviceAccountRoles, &awscloudcontrollermanager.ServiceAccount{})
+			}
 		}
 		if b.Cluster.Spec.CloudConfig != nil && b.Cluster.Spec.CloudConfig.AWSEBSCSIDriver != nil && fi.BoolValue(b.Cluster.Spec.CloudConfig.AWSEBSCSIDriver.Enabled) {
 			key := "aws-ebs-csi-driver.addons.k8s.io"
 
-			version := "1.0.0-kops.1"
 			{
 				id := "k8s-1.17"
 				location := key + "/" + id + ".yaml"
 				addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-					Name:              fi.String(key),
-					Version:           fi.String(version),
-					Manifest:          fi.String(location),
-					Selector:          map[string]string{"k8s-addon": key},
-					KubernetesVersion: ">=1.17.0",
-					Id:                id,
+					Name:     fi.String(key),
+					Manifest: fi.String(location),
+					Selector: map[string]string{"k8s-addon": key},
+					Id:       id,
 				})
 			}
+
+			// Generate aws-load-balancer-controller ServiceAccount IAM permissions
+			if b.UseServiceAccountExternalPermissions() {
+				serviceAccountRoles = append(serviceAccountRoles, &awsebscsidriver.ServiceAccount{})
+			}
 		}
 	}
 
-	if b.Cluster.Spec.SnapshotController != nil && fi.BoolValue(b.Cluster.Spec.SnapshotController.Enabled) {
+	if b.IsKubernetesGTE("1.20") && b.Cluster.Spec.SnapshotController != nil && fi.BoolValue(b.Cluster.Spec.SnapshotController.Enabled) {
 		key := "snapshot-controller.addons.k8s.io"
 
-		version := "4.0.0-kops.1"
 		{
 			id := "k8s-1.20"
 			location := key + "/" + id + ".yaml"
 			addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
-				Name:              fi.String(key),
-				Version:           fi.String(version),
-				Manifest:          fi.String(location),
-				Selector:          map[string]string{"k8s-addon": key},
-				KubernetesVersion: ">=1.20.0",
-				NeedsPKI:          true,
-				Id:                id,
+				Name:     fi.String(key),
+				Manifest: fi.String(location),
+				Selector: map[string]string{"k8s-addon": key},
+				NeedsPKI: true,
+				Id:       id,
 			})
 		}
 	}
@@ -1068,11 +983,24 @@ func (b *BootstrapChannelBuilder) buildAddons(c *fi.ModelBuilderContext) (*chann
 
 		addons.Spec.Addons = append(addons.Spec.Addons, &channelsapi.AddonSpec{
 			Name:     fi.String(key),
-			Version:  fi.String(version),
 			Selector: map[string]string{"k8s-addon": key},
 			Manifest: fi.String(location),
 		})
 	}
 
+	if kops.CloudProviderID(b.Cluster.Spec.CloudProvider) == kops.CloudProviderAWS && b.Cluster.Spec.KubeAPIServer.ServiceAccountIssuer != nil {
+		awsModelContext := &awsmodel.AWSModelContext{
+			KopsModelContext: b.KopsModelContext,
+		}
+
+		for _, serviceAccountRole := range serviceAccountRoles {
+			iamModelBuilder := &awsmodel.IAMModelBuilder{AWSModelContext: awsModelContext, Lifecycle: b.Lifecycle, Cluster: b.Cluster}
+
+			_, err := iamModelBuilder.BuildServiceAccountRoleTasks(serviceAccountRole, c)
+			if err != nil {
+				return nil, err
+			}
+		}
+	}
 	return addons, nil
 }
