@@ -44,17 +44,18 @@ func TestBootstrapChannelBuilder_BuildTasks(t *testing.T) {
 
 	h.SetupMockAWS()
 
-	runChannelBuilderTest(t, "simple", []string{"dns-controller.addons.k8s.io-k8s-1.12", "kops-controller.addons.k8s.io-k8s-1.16"})
+	runChannelBuilderTest(t, "simple", []string{"kops-controller.addons.k8s.io-k8s-1.16"})
 	// Use cilium networking, proxy
-	runChannelBuilderTest(t, "cilium", []string{"dns-controller.addons.k8s.io-k8s-1.12", "kops-controller.addons.k8s.io-k8s-1.16"})
+	runChannelBuilderTest(t, "cilium", []string{"kops-controller.addons.k8s.io-k8s-1.16"})
 	runChannelBuilderTest(t, "weave", []string{})
-	runChannelBuilderTest(t, "amazonvpc", []string{"networking.amazon-vpc-routed-eni-k8s-1.12", "networking.amazon-vpc-routed-eni-k8s-1.16"})
-	runChannelBuilderTest(t, "amazonvpc-containerd", []string{"networking.amazon-vpc-routed-eni-k8s-1.12", "networking.amazon-vpc-routed-eni-k8s-1.16"})
+	runChannelBuilderTest(t, "amazonvpc", []string{"networking.amazon-vpc-routed-eni-k8s-1.16"})
+	runChannelBuilderTest(t, "amazonvpc-containerd", []string{"networking.amazon-vpc-routed-eni-k8s-1.16"})
 	runChannelBuilderTest(t, "awsiamauthenticator", []string{"authentication.aws-k8s-1.12"})
 	runChannelBuilderTest(t, "metrics-server/insecure-1.18", []string{"metrics-server.addons.k8s.io-k8s-1.11"})
 	runChannelBuilderTest(t, "metrics-server/insecure-1.19", []string{"metrics-server.addons.k8s.io-k8s-1.11"})
 	runChannelBuilderTest(t, "metrics-server/secure-1.18", []string{"metrics-server.addons.k8s.io-k8s-1.11"})
 	runChannelBuilderTest(t, "metrics-server/secure-1.19", []string{"metrics-server.addons.k8s.io-k8s-1.11"})
+	runChannelBuilderTest(t, "coredns", []string{"coredns.addons.k8s.io-k8s-1.12"})
 }
 
 func TestBootstrapChannelBuilder_ServiceAccountIAM(t *testing.T) {
@@ -63,9 +64,9 @@ func TestBootstrapChannelBuilder_ServiceAccountIAM(t *testing.T) {
 
 	h.SetupMockAWS()
 
-	featureflag.ParseFlags("+UseServiceAccountIAM")
+	featureflag.ParseFlags("+UseServiceAccountExternalPermissions")
 	unsetFeatureFlag := func() {
-		featureflag.ParseFlags("-UseServiceAccountIAM")
+		featureflag.ParseFlags("-UseServiceAccountExternalPermissions")
 	}
 	defer unsetFeatureFlag()
 	runChannelBuilderTest(t, "service-account-iam", []string{"dns-controller.addons.k8s.io-k8s-1.12", "kops-controller.addons.k8s.io-k8s-1.16"})
@@ -77,11 +78,6 @@ func TestBootstrapChannelBuilder_AWSCloudController(t *testing.T) {
 
 	h.SetupMockAWS()
 
-	featureflag.ParseFlags("+EnableExternalCloudController")
-	unsetFeatureFlag := func() {
-		featureflag.ParseFlags("-EnableExternalCloudController")
-	}
-	defer unsetFeatureFlag()
 	runChannelBuilderTest(t, "awscloudcontroller", []string{"aws-cloud-controller.addons.k8s.io-k8s-1.18"})
 }
 
@@ -164,8 +160,8 @@ func runChannelBuilderTest(t *testing.T, key string, addonManifests []string) {
 
 	bcb := bootstrapchannelbuilder.NewBootstrapChannelBuilder(
 		&kopsModel,
-		nil,
-		assets.NewAssetBuilder(cluster, ""),
+		fi.LifecycleSync,
+		assets.NewAssetBuilder(cluster, false),
 		templates,
 		nil,
 	)
