@@ -23,8 +23,7 @@ import (
 	"strings"
 	"time"
 
-	"github.com/gophercloud/gophercloud/openstack/compute/v2/flavors"
-
+	"github.com/blang/semver/v4"
 	"github.com/gophercloud/gophercloud"
 	os "github.com/gophercloud/gophercloud/openstack"
 	cinder "github.com/gophercloud/gophercloud/openstack/blockstorage/v3/volumes"
@@ -32,10 +31,12 @@ import (
 	"github.com/gophercloud/gophercloud/openstack/compute/v2/extensions/keypairs"
 	"github.com/gophercloud/gophercloud/openstack/compute/v2/extensions/servergroups"
 	"github.com/gophercloud/gophercloud/openstack/compute/v2/extensions/volumeattach"
+	"github.com/gophercloud/gophercloud/openstack/compute/v2/flavors"
 	"github.com/gophercloud/gophercloud/openstack/compute/v2/servers"
 	"github.com/gophercloud/gophercloud/openstack/dns/v2/recordsets"
 	"github.com/gophercloud/gophercloud/openstack/dns/v2/zones"
 	"github.com/gophercloud/gophercloud/openstack/imageservice/v2/images"
+	"github.com/gophercloud/gophercloud/openstack/loadbalancer/v2/apiversions"
 	"github.com/gophercloud/gophercloud/openstack/loadbalancer/v2/listeners"
 	"github.com/gophercloud/gophercloud/openstack/loadbalancer/v2/loadbalancers"
 	"github.com/gophercloud/gophercloud/openstack/loadbalancer/v2/monitors"
@@ -64,8 +65,10 @@ const (
 	TagNameRolePrefix        = "k8s.io/role/"
 	TagClusterName           = "KubernetesCluster"
 	TagRoleMaster            = "master"
+	TagKopsInstanceGroup     = "KopsInstanceGroup"
 	TagKopsNetwork           = "KopsNetwork"
 	TagKopsName              = "KopsName"
+	TagKopsRole              = "KopsRole"
 	ResourceTypePort         = "ports"
 	ResourceTypeNetwork      = "networks"
 	ResourceTypeSubnet       = "subnets"
@@ -264,10 +267,6 @@ type OpenstackCloud interface {
 	// DeleteLB will delete loadbalancer
 	DeleteLB(lbID string, opt loadbalancers.DeleteOpts) error
 
-	GetApiIngressStatus(cluster *kops.Cluster) ([]kops.ApiIngressStatus, error)
-
-	FindClusterStatus(cluster *kops.Cluster) (*kops.ClusterStatus, error)
-
 	// DefaultInstanceType determines a suitable instance type for the specified instance group
 	DefaultInstanceType(cluster *kops.Cluster, ig *kops.InstanceGroup) (string, error)
 
@@ -312,6 +311,8 @@ type OpenstackCloud interface {
 	CreateL3FloatingIP(opts l3floatingip.CreateOpts) (fip *l3floatingip.FloatingIP, err error)
 	DeleteFloatingIP(id string) error
 	DeleteL3FloatingIP(id string) error
+
+	UseLoadBalancerVIPACL() (bool, error)
 }
 
 type openstackCloud struct {
@@ -329,6 +330,7 @@ type openstackCloud struct {
 	useOctavia      bool
 	zones           []string
 	floatingEnabled bool
+	useVIPACL       *bool
 }
 
 var _ fi.Cloud = &openstackCloud{}
@@ -659,16 +661,48 @@ func (c *openstackCloud) GetCloudTags() map[string]string {
 	return c.tags
 }
 
+func (c *openstackCloud) UseLoadBalancerVIPACL() (bool, error) {
+	if c.useVIPACL != nil {
+		return *c.useVIPACL, nil
+	}
+	use, err := useLoadBalancerVIPACL(c)
+	if err != nil {
+		return false, err
+	}
+	c.useVIPACL = &use
+	return use, nil
+}
+
+func useLoadBalancerVIPACL(c OpenstackCloud) (bool, error) {
+	allPages, err := apiversions.List(c.LoadBalancerClient()).AllPages()
+	if err != nil {
+		return false, err
+	}
+	versions, err := apiversions.ExtractAPIVersions(allPages)
+	if err != nil {
+		return false, err
+	}
+	if len(versions) == 0 {
+		return false, fmt.Errorf("loadbalancer API versions not found")
+	}
+	ver, err := semver.ParseTolerant(versions[len(versions)-1].ID)
+	if err != nil {
+		return false, err
+	}
+	// https://github.com/kubernetes/cloud-provider-openstack/blob/721615aa256bbddbd481cfb4a887c3ab180c5563/pkg/util/openstack/loadbalancer.go#L108
+	return ver.Compare(semver.MustParse("2.12.0")) > 0, nil
+}
+
 type Address struct {
 	IPType string `mapstructure:"OS-EXT-IPS:type"`
 	Addr   string
 }
 
-func (c *openstackCloud) GetApiIngressStatus(cluster *kops.Cluster) ([]kops.ApiIngressStatus, error) {
+func (c *openstackCloud) GetApiIngressStatus(cluster *kops.Cluster) ([]fi.ApiIngressStatus, error) {
 	return getApiIngressStatus(c, cluster)
 }
 
-func getApiIngressStatus(c OpenstackCloud, cluster *kops.Cluster) ([]kops.ApiIngressStatus, error) {
+func getApiIngressStatus(c OpenstackCloud, cluster *kops.Cluster) ([]fi.ApiIngressStatus, error) {
 	if cluster.Spec.CloudConfig.Openstack.Loadbalancer != nil {
 		return getLoadBalancerIngressStatus(c, cluster)
 	} else {
@@ -676,8 +710,8 @@ func getApiIngressStatus(c OpenstackCloud, cluster *kops.Cluster) ([]kops.ApiIng
 	}
 }
 
-func getLoadBalancerIngressStatus(c OpenstackCloud, cluster *kops.Cluster) ([]kops.ApiIngressStatus, error) {
-	var ingresses []kops.ApiIngressStatus
+func getLoadBalancerIngressStatus(c OpenstackCloud, cluster *kops.Cluster) ([]fi.ApiIngressStatus, error) {
+	var ingresses []fi.ApiIngressStatus
 	if cluster.Spec.MasterPublicName != "" {
 		// Note that this must match OpenstackModel lb name
 		klog.V(2).Infof("Querying Openstack to find Loadbalancers for API (%q)", cluster.Name)
@@ -697,7 +731,7 @@ func getLoadBalancerIngressStatus(c OpenstackCloud, cluster *kops.Cluster) ([]ko
 			}
 			for _, fip := range fips {
 				if fip.PortID == lb.VipPortID {
-					ingresses = append(ingresses, kops.ApiIngressStatus{
+					ingresses = append(ingresses, fi.ApiIngressStatus{
 						IP: fip.FloatingIP,
 					})
 				}
@@ -707,7 +741,7 @@ func getLoadBalancerIngressStatus(c OpenstackCloud, cluster *kops.Cluster) ([]ko
 	return ingresses, nil
 }
 
-func getIPIngressStatus(c OpenstackCloud, cluster *kops.Cluster) (ingresses []kops.ApiIngressStatus, err error) {
+func getIPIngressStatus(c OpenstackCloud, cluster *kops.Cluster) (ingresses []fi.ApiIngressStatus, err error) {
 	done, err := vfs.RetryWithBackoff(readBackoff, func() (bool, error) {
 		instances, err := c.ListInstances(servers.ListOpts{})
 		if err != nil {
@@ -725,7 +759,7 @@ func getIPIngressStatus(c OpenstackCloud, cluster *kops.Cluster) (ingresses []ko
 						if err != nil {
 							return false, fmt.Errorf("failed to get interface address: %v", err)
 						}
-						ingresses = append(ingresses, kops.ApiIngressStatus{
+						ingresses = append(ingresses, fi.ApiIngressStatus{
 							IP: address,
 						})
 					} else {
@@ -734,7 +768,7 @@ func getIPIngressStatus(c OpenstackCloud, cluster *kops.Cluster) (ingresses []ko
 							return false, err
 						}
 						for _, ip := range ips {
-							ingresses = append(ingresses, kops.ApiIngressStatus{
+							ingresses = append(ingresses, fi.ApiIngressStatus{
 								IP: fi.StringValue(ip),
 							})
 						}
