@@ -21,6 +21,8 @@ import (
 	"strings"
 	"testing"
 
+	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 	"k8s.io/kops/pkg/testutils"
 
 	kopsapi "k8s.io/kops/pkg/apis/kops"
@@ -38,6 +40,7 @@ func buildMinimalCluster() (*awsup.MockAWSCloud, *kopsapi.Cluster) {
 
 	return cloud, c
 }
+
 func TestPopulateCluster_Default_NoError(t *testing.T) {
 	cloud, c := buildMinimalCluster()
 
@@ -52,10 +55,69 @@ func TestPopulateCluster_Default_NoError(t *testing.T) {
 	}
 }
 
+func TestPopulateCluster_Subnets(t *testing.T) {
+	tests := []struct {
+		NonMasqueradeCIDR             string
+		ExpectedClusterCIDR           string
+		ExpectedServiceClusterIPRange string
+	}{
+		{
+			NonMasqueradeCIDR:             "100.64.0.0/10",
+			ExpectedClusterCIDR:           "100.96.0.0/11",
+			ExpectedServiceClusterIPRange: "100.64.0.0/13",
+		},
+		{
+			NonMasqueradeCIDR:             "10.0.0.0/9",
+			ExpectedClusterCIDR:           "10.64.0.0/10",
+			ExpectedServiceClusterIPRange: "10.0.0.0/12",
+		},
+		{
+			NonMasqueradeCIDR:             "10.0.0.0/8",
+			ExpectedClusterCIDR:           "10.128.0.0/9",
+			ExpectedServiceClusterIPRange: "10.0.0.0/12",
+		},
+		{
+			NonMasqueradeCIDR:             "fd00:10:96::/96",
+			ExpectedClusterCIDR:           "fd00:10:96::8000:0/97",
+			ExpectedServiceClusterIPRange: "fd00:10:96::/108",
+		},
+		{
+			NonMasqueradeCIDR:             "fd00:10:96::/95",
+			ExpectedClusterCIDR:           "fd00:10:96::1:0:0/96",
+			ExpectedServiceClusterIPRange: "fd00:10:96::/108",
+		},
+		{
+			NonMasqueradeCIDR:             "fd00:10:96::/94",
+			ExpectedClusterCIDR:           "fd00:10:96::1:0:0/96",
+			ExpectedServiceClusterIPRange: "fd00:10:96::/108",
+		},
+		{
+			NonMasqueradeCIDR:             "fd00:10:96::/106",
+			ExpectedClusterCIDR:           "fd00:10:96::20:0/107",
+			ExpectedServiceClusterIPRange: "fd00:10:96::/109",
+		},
+	}
+	for _, tc := range tests {
+		t.Run(tc.NonMasqueradeCIDR, func(t *testing.T) {
+			cloud, c := buildMinimalCluster()
+			c.Spec.NonMasqueradeCIDR = tc.NonMasqueradeCIDR
+
+			err := PerformAssignments(c, cloud)
+			require.NoError(t, err, "PerformAssignments")
+
+			full, err := mockedPopulateClusterSpec(c, cloud)
+			require.NoError(t, err, "PopulateClusterSpec")
+
+			assert.Equal(t, tc.ExpectedClusterCIDR, full.Spec.KubeControllerManager.ClusterCIDR, "ClusterCIDR")
+			assert.Equal(t, tc.ExpectedServiceClusterIPRange, full.Spec.ServiceClusterIPRange, "ServiceClusterIPRange")
+		})
+	}
+}
+
 func mockedPopulateClusterSpec(c *kopsapi.Cluster, cloud fi.Cloud) (*kopsapi.Cluster, error) {
 	vfs.Context.ResetMemfsContext(true)
 
-	assetBuilder := assets.NewAssetBuilder(c, "")
+	assetBuilder := assets.NewAssetBuilder(c, false)
 	basePath, err := vfs.Context.BuildVfsPath("memfs://tests")
 	if err != nil {
 		return nil, fmt.Errorf("error building vfspath: %v", err)
@@ -350,7 +412,7 @@ func TestPopulateCluster_APIServerCount(t *testing.T) {
 
 func TestPopulateCluster_AnonymousAuth(t *testing.T) {
 	cloud, c := buildMinimalCluster()
-	c.Spec.KubernetesVersion = "1.15.0"
+	c.Spec.KubernetesVersion = "1.20.0"
 
 	err := PerformAssignments(c, cloud)
 	if err != nil {
@@ -376,18 +438,6 @@ func TestPopulateCluster_DockerVersion(t *testing.T) {
 		KubernetesVersion string
 		DockerVersion     string
 	}{
-		{
-			KubernetesVersion: "1.13.0",
-			DockerVersion:     "18.06.3",
-		},
-		{
-			KubernetesVersion: "1.15.6",
-			DockerVersion:     "18.06.3",
-		},
-		{
-			KubernetesVersion: "1.16.0",
-			DockerVersion:     "18.09.9",
-		},
 		{
 			KubernetesVersion: "1.17.0",
 			DockerVersion:     "19.03.15",
