@@ -55,6 +55,7 @@ import (
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
 	"k8s.io/kops/upup/pkg/fi/cloudup/gce"
 	"k8s.io/kops/util/pkg/env"
+	"sigs.k8s.io/yaml"
 )
 
 // TemplateFunctions provides a collection of methods used throughout the templates
@@ -73,6 +74,7 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 	dest["EtcdScheme"] = tf.EtcdScheme
 	dest["SharedVPC"] = tf.SharedVPC
 	dest["ToJSON"] = tf.ToJSON
+	dest["ToYAML"] = tf.ToYAML
 	dest["UseBootstrapTokens"] = tf.UseBootstrapTokens
 	dest["UseEtcdTLS"] = tf.UseEtcdTLS
 	// Remember that we may be on a different arch from the target.  Hard-code for now.
@@ -99,6 +101,7 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 	dest["GetNodeInstanceGroups"] = tf.GetNodeInstanceGroups
 	dest["HasHighlyAvailableControlPlane"] = tf.HasHighlyAvailableControlPlane
 	dest["ControlPlaneControllerReplicas"] = tf.ControlPlaneControllerReplicas
+	dest["APIServerNodeRole"] = tf.APIServerNodeRole
 
 	dest["CloudTags"] = tf.CloudTagsForInstanceGroup
 	dest["KubeDNS"] = func() *kops.KubeDNSConfig {
@@ -128,7 +131,6 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 
 	// will return openstack external ccm image location for current kubernetes version
 	dest["OpenStackCCMTag"] = tf.OpenStackCCMTag
-	dest["AWSCCMTag"] = tf.AWSCCMTag
 	dest["ProxyEnv"] = tf.ProxyEnv
 
 	dest["KopsSystemEnv"] = tf.KopsSystemEnv
@@ -149,6 +151,19 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 		}
 	}
 
+	if cluster.Spec.Networking != nil && cluster.Spec.Networking.AmazonVPC != nil {
+		c := cluster.Spec.Networking.AmazonVPC
+		dest["AmazonVpcEnvVars"] = func() map[string]string {
+			envVars := map[string]string{
+				"AWS_VPC_K8S_CNI_CONFIGURE_RPFILTER": "false",
+			}
+			for _, e := range c.Env {
+				envVars[e.Name] = e.Value
+			}
+			return envVars
+		}
+	}
+
 	if cluster.Spec.Networking != nil && cluster.Spec.Networking.Calico != nil {
 		c := cluster.Spec.Networking.Calico
 		dest["CalicoIPv4PoolIPIPMode"] = func() string {
@@ -158,19 +173,19 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 			if c.IPIPMode != "" {
 				return c.IPIPMode
 			}
-			if c.CrossSubnet {
-				return "CrossSubnet"
+			if kops.CloudProviderID(cluster.Spec.CloudProvider) == kops.CloudProviderOpenstack {
+				return "Always"
 			}
-			return "Always"
+			return "CrossSubnet"
 		}
 		dest["CalicoIPv4PoolVXLANMode"] = func() string {
 			if c.EncapsulationMode != "vxlan" {
 				return "Never"
 			}
-			if c.CrossSubnet {
-				return "CrossSubnet"
+			if c.VXLANMode != "" {
+				return c.VXLANMode
 			}
-			return "Always"
+			return "CrossSubnet"
 		}
 	}
 
@@ -211,7 +226,7 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 		dest["WeaveSecret"] = func() string { return weavesecretString }
 	}
 
-	dest["CsiExtraTags"] = func() string {
+	dest["CloudLabels"] = func() string {
 		labels := []string{
 			fmt.Sprintf("KubernetesCluster=%s", cluster.ObjectMeta.Name),
 		}
@@ -223,7 +238,8 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 		return strings.Join(labels, ",")
 	}
 
-	dest["UseServiceAccountIAM"] = tf.UseServiceAccountIAM
+	dest["IsIPv6Only"] = tf.IsIPv6Only
+	dest["UseServiceAccountExternalPermissions"] = tf.UseServiceAccountExternalPermissions
 
 	if cluster.Spec.NodeTerminationHandler != nil {
 		dest["DefaultQueueName"] = func() string {
@@ -252,6 +268,16 @@ func (tf *TemplateFunctions) ToJSON(data interface{}) string {
 	return string(encoded)
 }
 
+// ToYAML returns a yaml representation of the struct or on error an empty string
+func (tf *TemplateFunctions) ToYAML(data interface{}) string {
+	encoded, err := yaml.Marshal(data)
+	if err != nil {
+		return ""
+	}
+
+	return string(encoded)
+}
+
 // EtcdScheme parses and grabs the protocol to the etcd cluster
 func (tf *TemplateFunctions) EtcdScheme() string {
 	if tf.UseEtcdTLS() {
@@ -284,6 +310,13 @@ func (tf *TemplateFunctions) ControlPlaneControllerReplicas() int {
 	return 1
 }
 
+func (tf *TemplateFunctions) APIServerNodeRole() string {
+	if featureflag.APIServerNodes.Enabled() {
+		return "node-role.kubernetes.io/api-server"
+	}
+	return "node-role.kubernetes.io/master"
+}
+
 // HasHighlyAvailableControlPlane returns true of the cluster has more than one control plane node. False otherwise.
 func (tf *TemplateFunctions) HasHighlyAvailableControlPlane() bool {
 	cp := 0
@@ -343,6 +376,8 @@ func (tf *TemplateFunctions) CloudControllerConfigArgv() ([]string, error) {
 		argv = append(argv, fmt.Sprintf("--use-service-account-credentials=%t", true))
 	}
 
+	argv = append(argv, "--cloud-config=/etc/kubernetes/cloud.config")
+
 	return argv, nil
 }
 
@@ -556,6 +591,7 @@ func (tf *TemplateFunctions) KopsControllerArgv() ([]string, error) {
 
 func (tf *TemplateFunctions) ExternalDNSArgv() ([]string, error) {
 	cluster := tf.Cluster
+	externalDNS := tf.Cluster.Spec.ExternalDNS
 
 	var argv []string
 
@@ -572,7 +608,19 @@ func (tf *TemplateFunctions) ExternalDNSArgv() ([]string, error) {
 		return nil, fmt.Errorf("unhandled cloudprovider %q", cluster.Spec.CloudProvider)
 	}
 
-	argv = append(argv, "--source=ingress")
+	argv = append(argv, "--events")
+	if fi.BoolValue(externalDNS.WatchIngress) {
+		argv = append(argv, "--source=ingress")
+	}
+	argv = append(argv, "--source=pod")
+	argv = append(argv, "--source=service")
+	argv = append(argv, "--compatibility=kops-dns-controller")
+	argv = append(argv, "--registry=txt")
+	argv = append(argv, "--txt-owner-id=kops-"+tf.ClusterName())
+	argv = append(argv, "--zone-id-filter="+tf.Cluster.Spec.DNSZone)
+	if externalDNS.WatchNamespace != "" {
+		argv = append(argv, "--namespace="+externalDNS.WatchNamespace)
+	}
 
 	return argv, nil
 }
@@ -630,28 +678,6 @@ func (tf *TemplateFunctions) OpenStackCCMTag() string {
 	return tag
 }
 
-// AWSCCMTag returns the correct tag for the cloud controller manager based on
-// the Kubernetes Version
-func (tf *TemplateFunctions) AWSCCMTag() (string, error) {
-	var tag string
-	parsed, err := util.ParseKubernetesVersion(tf.Cluster.Spec.KubernetesVersion)
-	if err != nil {
-		return "", fmt.Errorf("failed to parse Kubernetes version from cluster spec: %q", err)
-	}
-
-	// Update when we have stable releases
-	switch parsed.Minor {
-	case 18:
-		tag = "v1.18.0-alpha.1"
-	case 19:
-		tag = "v1.19.0-alpha.1"
-	default:
-		tag = "latest"
-	}
-
-	return tag, nil
-}
-
 // GetNodeInstanceGroups returns a map containing the defined instance groups of role "Node".
 func (tf *TemplateFunctions) GetNodeInstanceGroups() map[string]kops.InstanceGroupSpec {
 	nodegroups := make(map[string]kops.InstanceGroupSpec)
