@@ -114,6 +114,11 @@ func TestFindDistribution(t *testing.T) {
 			err:      nil,
 			expected: DistributionUbuntu2104,
 		},
+		{
+			rootfs:   "ubuntu2110",
+			err:      nil,
+			expected: DistributionUbuntu2110,
+		},
 		{
 			rootfs:   "notfound",
 			err:      fmt.Errorf("reading /etc/os-release: open tests/notfound/etc/os-release: no such file or directory"),
